Name: intel-igb
Summary: Intel(R) Gigabit Ethernet Connection
Version: 3.1.16
Release: 1
Source: %{name}-%{version}.tar.gz
Vendor: Intel Corporation
License: GPL
ExclusiveOS: linux
Group: System Environment/Kernel
Provides: %{name}
BuildRequires: pciutils-ids
URL: http://support.intel.com
BuildRoot: %{_tmppath}/%{name}-%{version}-root
# do not generate debugging packages by default - newer versions of rpmbuild
# may instead need:
#%define debug_package %{nil}
%debug_package %{nil}
# macros for finding system files to update at install time (pci.ids, pcitable)
%define find() %(for f in %*; do if [ -e $f ]; then echo $f; break; fi; done)
%define _pciids   /usr/share/pci.ids        /usr/share/hwdata/pci.ids
%define _pcitable /usr/share/kudzu/pcitable /usr/share/hwdata/pcitable /dev/null
%define pciids    %find %{_pciids}
%define pcitable  %find %{_pcitable}
Requires: kernel, fileutils, findutils, gawk, bash

%description
This package contains the Linux driver for the Intel(R) Gigabit Family of Server Adapters.

%package KMP
Summary: igb Kernel Module
Group: System/Kernel

%description KMP
This is one of the sub-packages for a specific kernel.  All the
sub-packages will share the same summary, group, and description.

%prep
%setup

%build
mkdir -p %{buildroot}

KV=$(uname -r)
KA=%{_arch}
KV_BASE=$(echo $KV | sed '{ s/hugemem//g; s/smp//g; s/enterprise//g; }' )

if [ -e /usr/src/kernels ] && [ $(echo $KV_BASE | grep "^2.6") ]; then
	if [ -e /etc/redhat-release ]; then
		KSP=$(ls /lib/modules | grep $KV_BASE)
		for K in $KSP ; do
			if [ $KA == "x86_64" ] && \
			   [ $(echo $K | grep hugemem) ]; then
				# Include path for x86_64 hugemem is broken
				# on RHEL4
				continue
			fi
			make -C src clean
			make -C src KSP=/lib/modules/$K/build \
				INSTALL_MOD_PATH=%{buildroot} \
				KVERSION=$k \
				MANDIR=%{_mandir} \
				CFLAGS_EXTRA="$CFLAGS_EXTRA" install
		done
	else
		make -C src clean
		make -C src INSTALL_MOD_PATH=%{buildroot} \
			MANDIR=%{_mandir} install
	fi
else
	SwitchRHKernel () {
		CFLAGS_EXTRA=""
		for K in $2 ; do
			if [ $K == $1 ] ; then
				CFLAGS_EXTRA="$CFLAGS_EXTRA -D__BOOT_KERNEL_$K=1"
			else
				CFLAGS_EXTRA="$CFLAGS_EXTRA -D__BOOT_KERNEL_$K=0"
			fi
		done
	}

	KSP="/lib/modules/$KV/build
	     /usr/src/linux-$KV
	     /usr/src/linux-$(echo $KV | sed 's/-.*//')
	     /usr/src/kernel-headers-$KV
	     /usr/src/kernel-source-$KV
	     /usr/src/linux-$(echo $KV | sed 's/\([0-9]*\.[0-9]*\)\..*/\1/')
	     /usr/src/linux"

	KSRC=$(for d in $KSP ; do [ -e $d/include/linux ] && echo $d; echo;  done)
	KSRC=$(echo $KSRC | awk '{ print $1 }')

	if [ -e $KSRC/include/linux/rhconfig.h ] ; then
		RHKL=$(grep 'BOOT_KERNEL_.* [01]' /boot/kernel.h |
		       sed 's/.*BOOT_KERNEL_\(.*\) [01]/\1/')
		if echo $RHKL | grep BIGMEM
		then
			RHKL=$(echo $RHKL | sed 's/ENTERPRISE//')
		fi
		if echo $RHKL | grep HUGEMEM
		then
			RHKL=$(echo $RHKL | sed 's/BIGMEM//')
		fi
		for K in $RHKL ; do
			SwitchRHKernel $K "$RHKL"
			make -C src clean
			if [ $KA == "x86_64" ] ; then
				CFLAGS_EXTRA="$CFLAGS_EXTRA -D__MODULE_KERNEL_x86_64=0 -D__MODULE_KERNEL_ia32e=1"
			fi
			make -C src INSTALL_MOD_PATH=%{buildroot} \
				MANDIR=%{_mandir} CFLAGS_EXTRA="$CFLAGS_EXTRA" install
		done
	else
		make -C src clean
		make -C src INSTALL_MOD_PATH=%{buildroot} MANDIR=%{_mandir} install
	fi
fi

%install
# Append .new to driver name to avoid conflict with kernel RPM
cd %{buildroot}
find lib -name "igb.*o" -exec mv {} {}.new \; \
         -fprintf %{_builddir}/%{name}-%{version}/file.list "/%p.new\n"


%clean
rm -rf %{buildroot}

%files -f %{_builddir}/%{name}-%{version}/file.list
%defattr(-,root,root)
%{_mandir}/man7/igb.7.gz
%doc COPYING
%doc README
%doc file.list
%doc pci.updates

%post
FL="%{_docdir}/%{name}-%{version}/file.list
    %{_docdir}/%{name}/file.list"
FL=$(for d in $FL ; do if [ -e $d ]; then echo $d; break; fi;  done)

if [ -d /usr/local/lib/%{name} ]; then
	rm -rf /usr/local/lib/%{name}
fi
if [ -d /usr/local/share/%{name} ]; then
	rm -rf /usr/local/share/%{name}
fi

# Save old drivers (aka .o and .o.gz)
echo "original pci.ids saved in /usr/local/share/%{name}";
if [ "%{pcitable}" != "/dev/null" ]; then
	echo "original pcitable saved in /usr/local/share/%{name}";
fi
for k in $(sed 's/\/lib\/modules\/\([0-9a-zA-Z_\.\-]*\).*/\1/' $FL) ; 
do
	d_drivers=/lib/modules/$k
	d_usr=/usr/local/share/%{name}/$k
	mkdir -p $d_usr
	cd $d_drivers; find . -name %{name}.*o -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{name}_*.*o -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{name}.*o.gz -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{name}_*.*o.gz -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cp --parents %{pciids} /usr/local/share/%{name}/
	if [ "%{pcitable}" != "/dev/null" ]; then
		cp --parents %{pcitable} /usr/local/share/%{name}/
	fi
done

# Add driver link
for f in $(sed 's/\.new$//' $FL) ; do
	ln -f $f.new $f 
done

# Check if kernel version rpm was built on IS the same as running kernel
BK_LIST=$(sed 's/\/lib\/modules\/\([0-9a-zA-Z_\.\-]*\).*/\1/' $FL)
MATCH=no
for i in $BK_LIST
do
	if [ $(uname -r) == $i ] ; then
		MATCH=yes
		break
	fi
done
if [ $MATCH == no ] ; then
	echo -n "WARNING: Running kernel is $(uname -r).  "
	echo -n "RPM supports kernels (  "
	for i in $BK_LIST
	do
		echo -n "$i  "
	done
	echo ")"
fi

LD="%{_docdir}/%{name}";
if [ -d %{_docdir}/%{name}-%{version} ]; then
	LD="%{_docdir}/%{name}-%{version}";
fi

cp /pci.updates /usr/share/pci.ids.d/pci.ids.e1000;
merge-pciids

uname -r | grep BOOT || /sbin/depmod -a > /dev/null 2>&1 || true

%preun
# If doing RPM un-install
if [ $1 -eq 0 ] ; then
	FL="%{_docdir}/%{name}-%{version}/file.list
    		%{_docdir}/%{name}/file.list"
	FL=$(for d in $FL ; do if [ -e $d ]; then echo $d; break; fi;  done)

	# Remove driver link
	for f in $(sed 's/\.new$//' $FL) ; do
		rm -f $f
	done

	# Restore old drivers
	if [ -d /usr/local/share/%{name} ]; then
		cd /usr/local/share/%{name}; find . -name '%{name}.*o*' -exec cp --parents {} /lib/modules/ \;
		cd /usr/local/share/%{name}; find . -name '%{name}_*.*o*' -exec cp --parents {} /lib/modules/ \;
		rm -rf /usr/local/share/%{name}
	fi
fi

%postun
uname -r | grep BOOT || /sbin/depmod -a > /dev/null 2>&1 || true

