/*******************************************************************************

  Intel 10 Gigabit PCI Express Linux driver
  Copyright(c) 1999 - 2011 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  e1000-devel Mailing List <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include "vmkapi.h"
#include "ixgbe.h"
#include "ixgbe_npa.h"

/************************Misc. APIs*************************/
#define IXGBE_CHECK_VF_ALLOC_STATE(_vf)					     \
	{								     \
		do {							     \
			struct ixgbe_adapter *adapter = netdev_priv(netdev); \
			if (vf >= adapter->num_vfs) {			     \
				DPRINTK(PROBE, INFO, "Invalid VF ID\n");     \
				return VMK_NOT_FOUND;			     \
			}						     \
			if (!adapter->vfinfo[vf].allocated) {		     \
				DPRINTK(PROBE, INFO, "VF not allocated\n");  \
				return VMK_NOT_READY;			     \
			}						     \
		} while (0);						     \
	}

#define IXGBE_CHECK_VF_INIT_STATE(_vf)					     \
	{								     \
		do {							     \
			IXGBE_CHECK_VF_ALLOC_STATE(_vf);		     \
			if (!adapter->vfinfo[vf].init) {		     \
				DPRINTK(PROBE, INFO, "VF not initialized\n");\
				return VMK_NOT_READY;			     \
			}						     \
		} while (0);						     \
	}


void ixgbe_vf_print_eswitch_regs(struct ixgbe_adapter *adapter)
{
	struct ixgbe_hw *hw = &adapter->hw;
	int i;
	u32 regl, regh;
	u32 dtxswc;
	u32 reg;


	DPRINTK(VIRT, INFO, "******** eSwitch Registers start ********\n");

	/* print active VFs */
	DPRINTK(VIRT, INFO, "*****Active VFs*****\n");
	DPRINTK(VIRT, INFO, "VFRE %08X:%08X VFTE %08X:%08X\n",
			IXGBE_READ_REG(hw, IXGBE_VFRE(1)),
			IXGBE_READ_REG(hw, IXGBE_VFRE(0)),
			IXGBE_READ_REG(hw, IXGBE_VFTE(1)),
			IXGBE_READ_REG(hw, IXGBE_VFTE(0)));

	/* print perfact filters */
	for (i = 0; i < hw->mac.rar_highwater; i++) {
		regl = IXGBE_READ_REG(hw, IXGBE_RAL(i));
		regh = IXGBE_READ_REG(hw, IXGBE_RAH(i));
		DPRINTK(VIRT, INFO, "Valid %d Pool %08X:%08X Addr %08X:%08X\n",
			((regh & IXGBE_RAH_AV) ? 1 : 0),
			IXGBE_READ_REG(hw, IXGBE_MPSAR_HI(i)),
			IXGBE_READ_REG(hw, IXGBE_MPSAR_LO(i)),
			(regh & ~(IXGBE_RAH_AV)),
			regl);
	}

	/* print UTA */

	/* print MTA */

	/* print VLVFs */
	DPRINTK(VIRT, INFO, "*****VLAN Filters*****\n");
	for (i = 0; i < IXGBE_VLVF_ENTRIES; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_VLVF(i));
		DPRINTK(VIRT, INFO, "Valid %d Pool %08X:%08X Vlan ID %08X\n",
			((reg & IXGBE_VLVF_VIEN) ? 1 : 0),
			IXGBE_READ_REG(hw, IXGBE_VLVFB((2*i) + 1)),
			IXGBE_READ_REG(hw, IXGBE_VLVFB(2*i)),
			(reg & IXGBE_VLVF_VLANID_MASK));
	}

	/* print default VLANs setings */
	DPRINTK(VIRT, INFO, "*****Default VLANs*****\n");
	for (i = 0; i < (adapter->num_vfs + adapter->num_rx_pools); i++) {
		DPRINTK(VIRT, INFO, "%08X\n",
			IXGBE_READ_REG(hw, IXGBE_VMVIR(i)));
	}

	/* print MAC/VLAN anti-spoofing settings */
	DPRINTK(VIRT, INFO, "*****Anti-spoofing*****\n");
	for (i = 0; i < 8; i++) {
		dtxswc = IXGBE_READ_REG(hw, IXGBE_DTXSWC(i));
		DPRINTK(VIRT, INFO, "VLAN Pool %08X MAC Pool %08X\n",
			(dtxswc & IXGBE_DTXSWC_VLAN_SPOOF_MASK),
			(dtxswc & IXGBE_DTXSWC_MAC_SPOOF_MASK));
	}

	/* print Mirroring settings */

	DPRINTK(VIRT, INFO, "******** eSwitch Registers end ********\n");
}

void ixgbe_vf_print_regs(struct ixgbe_adapter *adapter, u32 vf)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u64 num_gorc;
	u64 num_gotc;
	u32 vf_idx;
	struct vf_pf_shared_mem *shared_mem = 0;

	DPRINTK(VIRT, INFO, "******* VF %d Registers start *******\n", vf);
	DPRINTK(VIRT, INFO,
		"RDBAH %08X RDBAL %08X LEN %08X RDH %08X RDT %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVFRDBAHn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFRDBALn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFRDLENn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFRDHn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFRDTn(adapter, vf, 0)));
	DPRINTK(VIRT, INFO,
		"RXDCTL %08X SRRCTL %08X PSRTYPE %08X VMOLR %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFSRRCTLn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFPSRTYPE(vf)),
		IXGBE_READ_REG(hw, IXGBE_VMOLR(vf)));

	DPRINTK(VIRT, INFO,
		"TDBAH %08X TDBAL %08X LEN %08X TDH %08X TDT %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVFTDBAHn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFTDBALn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFTDLENn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFTDHn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFTDTn(adapter, vf, 0)));
	DPRINTK(VIRT, INFO, "TDWBAH %08X TDWBAL %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVFTDWBAHn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFTDWBALn(adapter, vf, 0)));
	DPRINTK(VIRT, INFO, "TXDCTL %08X DCA_TXCTRL %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVFDCA_TXCTRLn(adapter, vf, 0)));

	DPRINTK(VIRT, INFO,
		"EIMS %08X EIMC %08X EIAC %08X EIAM %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVTEIMS(vf)),
		IXGBE_READ_REG(hw, IXGBE_PVTEIMC(vf)),
		IXGBE_READ_REG(hw, IXGBE_PVTEIAC(vf)),
		IXGBE_READ_REG(hw, IXGBE_PVTEIAM(vf)));

	DPRINTK(VIRT, INFO,
		"IVAR %08X EITR %08X\n",
		IXGBE_READ_REG(hw, IXGBE_PVTIVARn(adapter, vf, 0)),
		IXGBE_READ_REG(hw, IXGBE_PVTEITRn(adapter, vf, 0)));

	num_gorc = IXGBE_READ_REG(hw, IXGBE_PVFGORC_MSB(vf));
	num_gorc = (num_gorc << 32) | IXGBE_READ_REG(hw, IXGBE_PVFGORC_LSB(vf));
	num_gotc = IXGBE_READ_REG(hw, IXGBE_PVFGOTC_MSB(vf));
	num_gotc = (num_gotc << 32) | IXGBE_READ_REG(hw, IXGBE_PVFGOTC_LSB(vf));
	DPRINTK(VIRT, INFO, "gprc %u gptc %u mprc %u gorc %llu gotc %llu\n",
		IXGBE_READ_REG(hw, IXGBE_PVFGPRC(vf)),
		IXGBE_READ_REG(hw, IXGBE_PVFGPTC(vf)),
		IXGBE_READ_REG(hw, IXGBE_PVFGMPRC(vf)),
		num_gorc,
		num_gotc);

	/* print vf's soft settings */
	DPRINTK(VIRT, INFO,
		"allocated %u, buffer mode %u coml method %u\n",
		adapter->vfinfo[vf].allocated,
		adapter->vfinfo[vf].buffer_mode,
		adapter->vfinfo[vf].coml_method);
	DPRINTK(VIRT, INFO,
		"mtu %u, vlan count %u irq rate %u\n",
		adapter->vfinfo[vf].mtu,
		adapter->vfinfo[vf].vlans_enabled,
		adapter->vfinfo[vf].irq_rate);

	vf_idx = find_first_bit(&(adapter->active_vfs), adapter->num_vfs);
	while (vf_idx < adapter->num_vfs) {
		shared_mem = adapter->vfinfo[vf_idx].vf_pf_shared_mem;

		/* Check error counters in shared memory and print to log. */
		if (shared_mem != 0) {
			if (shared_mem->errors.exception_flags
				& VF_PF_EXCEPTION_FLAGS_DESC_ERROR)
				DPRINTK(VIRT, INFO, "Num Desc errors = %ul\n",
					shared_mem->errors.desc_errors);
			if (shared_mem->errors.exception_flags
				& VF_PF_EXCEPTION_FLAGS_LSO_ERROR)
				DPRINTK(VIRT, INFO, "Num LSO errors = %ul\n",
					shared_mem->errors.lso_errors);
		}
		vf_idx = find_next_bit(&(adapter->active_vfs),
					adapter->num_vfs,
					vf_idx + 1);
	}


	DPRINTK(VIRT, INFO, "******* VF %d Registers end ********\n", vf);
}

/* This routine checks to see if there are pools other than the default pool
 * which are still members of a VLAN. If not, it clears the default pool from
 * the VLAN membership, and cleans up the VLVF entry for the VLAN as well. If not,
 * both the VLVF entry and the pool membership is maintained as before.
 * ASSUMPTIONS: DCB is enabled, and 20 VFs are enabled
 */
void ixgbe_vf_check_or_clear_default_vlvf(struct ixgbe_adapter *adapter, u32 vlan) {
	s32 vlvf_index;
	u32 vlvf_reg_val;
	u32 default_pool_mask;

 	default_pool_mask = 0xffffffff & (~(1 << VMDQ_P(0)));
	vlvf_index = ixgbe_find_vlvf_slot (&adapter->hw, vlan);
	vlvf_reg_val = IXGBE_READ_REG(&adapter->hw, IXGBE_VLVFB(vlvf_index*2));
	vlvf_reg_val &= default_pool_mask;
	if (!vlvf_reg_val) {
		/* No other pools are members of this VLAN.
		 * Hence, clear VLVFB register, and remove VLVF entry.
		 */
		IXGBE_WRITE_REG(&adapter->hw, IXGBE_VLVFB(vlvf_index*2), 0);
		IXGBE_WRITE_REG(&adapter->hw, IXGBE_VLVF(vlvf_index), 0);
		DPRINTK(VIRT, INFO, "Clearing VLVF for vlan %d\n", vlan);
	}
	else
		DPRINTK(VIRT, INFO, "VLVF entry retained for vlan %d\n", vlan);
}

/*
 * callback routine for the vf allocation.  This routine is
 * one of the required parameter in the vmklnx_enable_vfs
 * function call, but it doesn't do any useful thing during its
 * invocation
 */
void ixgbe_npa_vfnotification_fn(struct pci_dev *pdev, struct pci_dev *vf,
				 int vf_idx, void *data)
{
	struct ixgbe_adapter *adapter = pci_get_drvdata(pdev);
	u32 enable = (u32)data;
	DPRINTK(VIRT, INFO, "vf idx = %d enable %u.\n", vf_idx, enable);
	return;
}

/* enable sr-iov mode */
int ixgbe_npa_enable_sriov(struct ixgbe_adapter *adapter)
{
	struct pci_dev *pdev = adapter->pdev;
	vmk_uint16 numvfs = adapter->num_vfs;
	int err = 0;

	DPRINTK(VIRT, INFO, "Allocate VFs = %d\n", numvfs);
	numvfs = vmklnx_enable_vfs(pdev, numvfs,
				   ixgbe_npa_vfnotification_fn,
				   (void *)1);
	if (numvfs) {
		adapter->num_vfs = numvfs;
		DPRINTK(PROBE, INFO, "Successfully enabled SR-IOV mode.\n");
	} else {
		DPRINTK(PROBE, INFO, "Unable to enable SR-IOV mode.\n");
		adapter->num_vfs = 0;
		err = -1;
	}

	DPRINTK(PROBE, INFO,
		"Allocated VFs = %d\n", adapter->num_vfs);
	return err;
}

/* enable sr-iov mode */
void ixgbe_npa_disable_sriov(struct ixgbe_adapter *adapter)
{
	struct pci_dev *pdev = adapter->pdev;
	int err;

	if (!adapter->num_vfs)
		return;

	vmklnx_disable_vfs(pdev, adapter->num_vfs,
			   ixgbe_npa_vfnotification_fn,
			   (void *)0);
	DPRINTK(PROBE, INFO, "Successfully disabled SR-IOV mode.\n");
	adapter->num_vfs = 0;
	return;
}

/* default configuration of the eswitch */
s32 ixgbe_npa_initialize(struct ixgbe_adapter *adapter)
{
	s32 ret = 0;
	int i, max_pools;

	bitmap_zero(adapter->active_vfs, IXGBE_MAX_VF_FUNCTIONS);
	if (adapter->flags & IXGBE_FLAG_DCB_ENABLED)
		max_pools = 32;
	else
		max_pools = 64;
	for (i = 0; i < max_pools; i++) {
		if (0 != ixgbe_vf_set_vlvfs(adapter, i, 0, 1))
			DPRINTK(PROBE, INFO, "Failed to add VLAN ID %u\n", i);
	}
	return ret;
}

/* cleanup the eswitch */
void ixgbe_npa_cleanup(struct ixgbe_adapter *adapter)
{
	DPRINTK(VIRT, INFO, "Cleaning up npa context\n");

	/* free memory for the VF state */
	if (adapter->vfinfo) {
		kfree(adapter->vfinfo);
		adapter->vfinfo = NULL;
	}
}

/* clear a given mac addr from the rars */
void ixgbe_vf_clear_addr(struct ixgbe_adapter *adapter, u32 vf, u8 *addr)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 rarl, rarh;
	u32 regl, regh;
	int i;

	DPRINTK(VIRT, INFO, "vf %u addr %02X:%02X:%02X:%02X:%02X:%02X\n",
		vf, addr[0], addr[1], addr[2],
		addr[3], addr[4], addr[5]);

	/*
	 * HW expects these in little endian so we reverse the byte order
	 * from network order (big endian) to little endian
	 */
	rarl = ((u32) addr[0] | ((u32) addr[1] << 8) |
		   ((u32) addr[2] << 16) | ((u32) addr[3] << 24));
	rarh = ((u32) addr[4] | ((u32) addr[5] << 8));

	for (i = 0; i < hw->mac.num_rar_entries; i++) {
		regl = IXGBE_READ_REG(hw, IXGBE_RAL(i));
		regh = IXGBE_READ_REG(hw, IXGBE_RAH(i));
		if ((rarl == regl) && (rarh == (regh & ~(IXGBE_RAH_AV)))) {
			ixgbe_clear_vmdq(hw, i, vf);
			adapter->vfinfo[vf].rar = -1;
			break;
		}
	}
}

/* insert a given mac addr to the rars for the default vf/pool */
s32 ixgbe_vf_insert_addr(struct ixgbe_adapter *adapter, u32 vf, u8 *addr)
{
	struct ixgbe_hw *hw = &adapter->hw;
	int ret = -1;
	int rar;

	DPRINTK(VIRT, INFO,
		"vf %u addr %02X:%02X:%02X:%02X:%02X:%02X\n",
		vf, addr[0], addr[1], addr[2],
		addr[3], addr[4], addr[5]);

	rar = ixgbe_insert_mac_addr(hw, addr, vf);
	if (rar >= 0) {
		adapter->vfinfo[vf].rar = rar;
		ret = 0;
	}

	return ret;
}

/* remove a given vf's membership from all the vlans */
void ixgbe_vf_clear_vlvfs(struct ixgbe_adapter *adapter, u32 vf)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 pool_mask, reg, vid;
	int i;

	DPRINTK(VIRT, INFO, "clear vlans for vf %u\n", vf);

	pool_mask = 1 << (vf%32);

	/* find the vlvfs with the matching vf id */
	for (i = 0; i < IXGBE_VLVF_ENTRIES; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_VLVFB(2*i + (vf/32)));
		if (reg & pool_mask){
			vid = IXGBE_READ_REG(hw, IXGBE_VLVF(i));
			vid = vid & IXGBE_VLVF_VLANID_MASK;
			/* keep vlan ID 0 to accept priority-tagged traffic */
			if (0 != vid)
				ixgbe_vf_set_vlvfs(adapter, vf, vid, 0);
		}
	}
}

/* add/remove vf's membership for a given vlan */
s32 ixgbe_vf_set_vlvfs(struct ixgbe_adapter *adapter, u32 vf,
		       u32 vid, bool add)
{
	struct ixgbe_hw *hw = &adapter->hw;

	DPRINTK(VIRT, INFO, "vf %u vlan id %u add %u\n", vf, vid, add);
	
	if (!vid && !add)
		return 0;

	/* following function has some functionality problems, needs
	to be fixed */
	return ixgbe_set_vfta(hw, vid, vf, add);
}

void ixgbe_vf_set_rlpml(struct ixgbe_adapter *adapter, u32 vf, u32 size)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 rxdctl;
	int i;

	DPRINTK(VIRT, INFO, "vf %u entry %u\n", vf, size);

}

void ixgbe_vf_unblock(struct ixgbe_adapter *adapter, u32 vf)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 vfre, vfte;

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	/* set the vfre/vfte & clear vflre of the vf */
	vfre = IXGBE_READ_REG(hw, IXGBE_VFRE(vf/32));
	vfte = IXGBE_READ_REG(hw, IXGBE_VFTE(vf/32));

	vfre |= (1 << (vf%32));
	vfte |= (1 << (vf%32));

	IXGBE_WRITE_REG(hw, IXGBE_VFLREC(vf/32), (1 << (vf%32)));
	IXGBE_WRITE_REG(hw, IXGBE_VFRE(vf/32), vfre);
	IXGBE_WRITE_REG(hw, IXGBE_VFTE(vf/32), vfte);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_block(struct ixgbe_adapter *adapter, u32 vf)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 vfre, vfte;

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	/* clear the vfre/vfte of the vf */
	vfre = IXGBE_READ_REG(hw, IXGBE_VFRE(vf/32));
	vfte = IXGBE_READ_REG(hw, IXGBE_VFTE(vf/32));

	vfre &= ~(1 << (vf%32));
	vfte &= ~(1 << (vf%32));

	IXGBE_WRITE_REG(hw, IXGBE_VFRE(vf/32), vfre);
	IXGBE_WRITE_REG(hw, IXGBE_VFTE(vf/32), vfte);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_reset(struct ixgbe_adapter *adapter, u32 vf)
{
	struct ixgbe_hw *hw = &adapter->hw;

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	/* reset the vf */
	IXGBE_WRITE_REG(hw, IXGBE_PVFCTRL(vf), IXGBE_CTRL_RST);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_read_stats(struct ixgbe_adapter *adapter, int vf,
			 struct vf_stats *pvf_stats)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u64 num;

	pvf_stats->gprc = IXGBE_READ_REG(hw, IXGBE_PVFGPRC(vf));
	pvf_stats->gptc = IXGBE_READ_REG(hw, IXGBE_PVFGPTC(vf));
	pvf_stats->mprc = IXGBE_READ_REG(hw, IXGBE_PVFGMPRC(vf));
	num = IXGBE_READ_REG(hw, IXGBE_PVFGORC_MSB(vf));
	pvf_stats->gorc = (num << 32) | IXGBE_READ_REG(hw, IXGBE_PVFGORC_LSB(vf));
	num = IXGBE_READ_REG(hw, IXGBE_PVFGOTC_MSB(vf));
	pvf_stats->gotc = (num << 32) | IXGBE_READ_REG(hw, IXGBE_PVFGOTC_LSB(vf));
}

/***********************VF Configuration APIs******************/

void ixgbe_vf_assign_vector(struct ixgbe_adapter *adapter, int vf,
			    int rx_queue, int tx_queue, int msix_vector)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 ivar, index;

	/* direction: 0 for Rx, 1 for Tx */
	msix_vector |= IXGBE_IVAR_ALLOC_VAL;
	index = ((16 * (rx_queue & 1)));
	ivar = IXGBE_READ_REG(hw, IXGBE_PVTIVARn(adapter, vf, (rx_queue >> 1)));
	ivar &= ~(0xFF << index);
	ivar |= (msix_vector << index);
	IXGBE_WRITE_REG(hw, IXGBE_PVTIVARn(adapter, vf, (rx_queue >> 1)), ivar);

	/* direction: 0 for Rx, 1 for Tx */
	msix_vector |= IXGBE_IVAR_ALLOC_VAL;
	index = ((16 * (tx_queue & 1)) + 8);
	ivar = IXGBE_READ_REG(hw, IXGBE_PVTIVARn(adapter, vf, (tx_queue >> 1)));
	ivar &= ~(0xFF << index);
	ivar |= (msix_vector << index);
	IXGBE_WRITE_REG(hw, IXGBE_PVTIVARn(adapter, vf, (tx_queue >> 1)), ivar);
}

void ixgbe_vf_configure_msix(struct ixgbe_adapter *adapter,
			     u32 vf, u32 num_vectors)
{
	struct ixgbe_hw *hw = &adapter->hw;
	int i;
	u32 mask;

	DPRINTK(VIRT, INFO, "vf %u num vectors %u\n", vf, num_vectors);

	for (i = 0; i < num_vectors; i++) {
		/* each queue pair is tied to a single vector */
		ixgbe_vf_assign_vector(adapter, vf, i, i, i);

		IXGBE_WRITE_REG(hw, IXGBE_PVTEITRn(adapter, vf, i),
			EITR_INTS_PER_SEC_TO_REG(adapter->vfinfo[vf].irq_rate));
	}

	mask = ((1 << num_vectors) - 1);
	IXGBE_WRITE_REG(hw, IXGBE_PVTEIAC(vf), mask);
	IXGBE_WRITE_REG(hw, IXGBE_PVTEIAM(vf), mask);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_configure_tx(struct ixgbe_adapter *adapter, u32 vf,
			   u32 queue_index, u64 tdba, u32 tdlen)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 txctrl, txdctl;

	DPRINTK(VIRT, INFO, "vf %u queue index %u, addr %llx len %u\n",
		vf, queue_index, tdba, tdlen);

	/* 128 bytes  (8 desc.) alignment */
	/* Setup the HW Tx Head and Tail descriptor pointers */
	IXGBE_WRITE_REG(hw, IXGBE_PVFTDBALn(adapter, vf, queue_index),
					(tdba & DMA_32BIT_MASK));
	IXGBE_WRITE_REG(hw, IXGBE_PVFTDBAHn(adapter, vf, queue_index),
					(tdba >> 32));
	IXGBE_WRITE_REG(hw, IXGBE_PVFTDLENn(adapter, vf, queue_index), tdlen);
	IXGBE_WRITE_REG(hw, IXGBE_PVFTDHn(adapter, vf, queue_index), 0);
	IXGBE_WRITE_REG(hw, IXGBE_PVFTDTn(adapter, vf, queue_index), 0);

	/* Disable Tx Head Writeback RO bit, since this hoses
	 * bookkeeping if things aren't delivered in order.
	 */
	txctrl = IXGBE_READ_REG(hw,
			IXGBE_PVFDCA_TXCTRLn(adapter, vf, queue_index));
	txctrl &= ~IXGBE_DCA_TXCTRL_TX_WB_RO_EN;
	IXGBE_WRITE_REG(hw,
			IXGBE_PVFDCA_TXCTRLn(adapter, vf, queue_index), txctrl);

	txdctl = IXGBE_READ_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, queue_index));
	if (VF_TX_COML_DD == adapter->vfinfo[vf].coml_method) {
		/* enable WTHRESH=8 descriptors, to encourage burst writeback */
		txdctl |= (8 << 16);
	} else if (VF_TX_COML_HEAD_WB == adapter->vfinfo[vf].coml_method) {
		u64 tdwba;
		tdwba = tdba + tdlen + IXGBE_TX_HEAD_DMA_SPACING;
		tdwba |= 1; /* enable head wb */
		IXGBE_WRITE_REG(hw, IXGBE_PVFTDWBALn(adapter, vf, queue_index),
				tdwba & 0x00000000ffffffffULL);
		IXGBE_WRITE_REG(hw, IXGBE_PVFTDWBAHn(adapter, vf, queue_index),
				tdwba >> 32);
	}
	IXGBE_WRITE_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, queue_index), txdctl);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_configure_srrctl(struct ixgbe_adapter *adapter, u32 vf,
			       u32 queue_index, u32 mtu)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 srrctl = 0;
	u16 rx_buf_len;
	int max_frame = mtu + ETH_HLEN + ETH_FCS_LEN;

	DPRINTK(VIRT, INFO, "vf %u queue index %u, mtu %u\n",
		vf, queue_index, mtu);

	if (mtu <= ETH_DATA_LEN)
		rx_buf_len = MAXIMUM_ETHERNET_VLAN_SIZE;
	else
		rx_buf_len = ALIGN(max_frame, 1024);

	srrctl = IXGBE_READ_REG(hw, IXGBE_PVFSRRCTLn(adapter, vf, queue_index));
	srrctl &= ~IXGBE_SRRCTL_BSIZEHDR_MASK;
	srrctl &= ~IXGBE_SRRCTL_BSIZEPKT_MASK;

	if (rx_buf_len > IXGBE_RXBUFFER_2048) {
		srrctl |= PAGE_SIZE >> IXGBE_SRRCTL_BSIZEPKT_SHIFT;
		srrctl |= IXGBE_SRRCTL_DESCTYPE_HDR_SPLIT_ALWAYS;
		srrctl |= (((IXGBE_MAX_HDR_SIZE) <<
			    IXGBE_SRRCTL_BSIZEHDRSIZE_SHIFT) &
			   IXGBE_SRRCTL_BSIZEHDR_MASK);

		/* ignore known traffic for the header-split */
		/* here the goal is to fillup hdr buffer upto the
		maximum length (960 bytes)*/
		IXGBE_WRITE_REG(hw, IXGBE_PVFPSRTYPE(vf), 0);
	} else {
		srrctl |= IXGBE_SRRCTL_DESCTYPE_ADV_ONEBUF;
		if (rx_buf_len == MAXIMUM_ETHERNET_VLAN_SIZE)
			srrctl |= IXGBE_RXBUFFER_2048 >>
				  IXGBE_SRRCTL_BSIZEPKT_SHIFT;
		else
			srrctl |= rx_buf_len >>
				  IXGBE_SRRCTL_BSIZEPKT_SHIFT;
	}

	/* Queue drop enable */
	srrctl |= IXGBE_SRRCTL_DROP_EN;
	IXGBE_WRITE_REG(hw, IXGBE_PVFSRRCTLn(adapter, vf, queue_index), srrctl);
}

void ixgbe_vf_configure_rx(struct ixgbe_adapter *adapter, u32 vf,
			   u32 queue_index,
			   u64 rdba, u32 rdlen,
			   u32 mtu)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 max_frame = mtu + ETH_HLEN + ETH_FCS_LEN;
	u32 vmolr = 0;
	u32 rxdctl = 0;

	DPRINTK(VIRT, INFO, "vf %u queue index %u, addr %llx len %u\n",
		vf, queue_index, rdba, rdlen);

	ixgbe_vf_configure_srrctl(adapter, vf, queue_index, mtu);

	IXGBE_WRITE_REG(hw, IXGBE_PVFRDBALn(adapter, vf, queue_index),
						(rdba & DMA_32BIT_MASK));
	IXGBE_WRITE_REG(hw, IXGBE_PVFRDBAHn(adapter, vf, queue_index),
						(rdba >> 32));
	IXGBE_WRITE_REG(hw, IXGBE_PVFRDLENn(adapter, vf, queue_index),
						rdlen);
	IXGBE_WRITE_REG(hw, IXGBE_PVFRDHn(adapter, vf, queue_index), 0);
	IXGBE_WRITE_REG(hw, IXGBE_PVFRDTn(adapter, vf, queue_index), 0);

	rxdctl = IXGBE_READ_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, queue_index));
	rxdctl |= IXGBE_RXDCTL_VME;
	IXGBE_WRITE_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, queue_index), rxdctl);

	vmolr = IXGBE_READ_REG(hw, IXGBE_VMOLR(vf));
	vmolr |= IXGBE_VMOLR_AUPE;
	IXGBE_WRITE_REG(hw, IXGBE_VMOLR(vf), vmolr);

	ixgbe_vf_set_rlpml(adapter, vf, max_frame);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_enable_queues(struct ixgbe_adapter *adapter,
			    u32 vf, u16 num_queue_pairs)
{
	struct ixgbe_hw *hw = &adapter->hw;
	int i;
	u32 reg = 0;
	/* rx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, i));
		reg |= IXGBE_RXDCTL_ENABLE;
		IXGBE_WRITE_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, i), reg);
	}

	/* tx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, i));
		reg |= IXGBE_TXDCTL_ENABLE;
		IXGBE_WRITE_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, i), reg);
	}

	/* make sure that queues are enabled */
	/* rx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		int loop_cnt = 10;
		do {
			msleep(1);
			reg = IXGBE_READ_REG(hw,
					     IXGBE_PVFRXDCTLn(adapter, vf, i));
			--loop_cnt;
		} while (!(reg & IXGBE_RXDCTL_ENABLE) && (loop_cnt >= 0));

		if (loop_cnt < 0)
			DPRINTK(PROBE, INFO, "vf %u rx queue %u not enabled\n",
				vf, i);
	}

	/* tx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		int loop_cnt = 10;
		do {
			msleep(1);
			reg = IXGBE_READ_REG(hw,
					     IXGBE_PVFTXDCTLn(adapter, vf, i));
			--loop_cnt;
		} while (!(reg & IXGBE_TXDCTL_ENABLE) && (loop_cnt >= 0));

		if (loop_cnt < 0)
			DPRINTK(PROBE, INFO, "vf %u tx queue %u not enabled\n",
				vf, i);
	}
}

void ixgbe_vf_disable_queues(struct ixgbe_adapter *adapter,
			     u32 vf, u16 num_queue_pairs)
{
	struct ixgbe_hw *hw = &adapter->hw;
	int i;
	u32 reg = 0;
	/* rx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, i));
		reg &= ~IXGBE_RXDCTL_ENABLE;
		IXGBE_WRITE_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, i), reg);
	}

	/* tx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		reg = IXGBE_READ_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, i));
		reg &= ~IXGBE_TXDCTL_ENABLE;
		IXGBE_WRITE_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, i), reg);
	}

	/* make sure that queues are disabled */
	/* rx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		int loop_cnt = 10;
		do {
			msleep(1);
			reg = IXGBE_READ_REG(hw,
					     IXGBE_PVFRXDCTLn(adapter, vf, i));
			--loop_cnt;
		} while ((reg & IXGBE_RXDCTL_ENABLE) && (loop_cnt >= 0));

		if (loop_cnt < 0)
			DPRINTK(PROBE, INFO, "vf %u rx queue %u not disabled\n",
				vf, i);
	}

	/* tx queues */
	for (i = 0; i < num_queue_pairs; i++) {
		int loop_cnt = 10;
		do {
			msleep(1);
			reg = IXGBE_READ_REG(hw,
					     IXGBE_PVFTXDCTLn(adapter, vf, i));
			--loop_cnt;
		} while ((reg & IXGBE_TXDCTL_ENABLE) && (loop_cnt >= 0));

		if (loop_cnt < 0)
			DPRINTK(PROBE, INFO, "vf %u tx queue %u not disabled\n",
				vf, i);
	}

}


/* disable all or a specific queue vector */
void ixgbe_vf_irq_disable(struct ixgbe_adapter *adapter, u32 vf, int v_idx)
{
	struct ixgbe_hw *hw = &adapter->hw;

	DPRINTK(VIRT, INFO, "vf %u vector %d\n", vf, v_idx);

	/* zero based index */
	if (v_idx < 0) {
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIAC(vf), 0);
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIMC(vf), ~0);
		IXGBE_WRITE_FLUSH(hw);
	} else {
		/* needs little work if we want individual irq disabled */
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIMC(vf), 1 << v_idx);
	}
	IXGBE_WRITE_FLUSH(hw);
}

/* enable all or a specific queue vector */
void ixgbe_vf_irq_enable(struct ixgbe_adapter *adapter, u32 vf, int v_idx)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 mask;

	DPRINTK(VIRT, INFO, "vf %u vector %d\n", vf, v_idx);

	/* zero based index */
	if (v_idx < 0) {
		mask = (1 << (IXGBE_MAX_VF_VECTORS)) - 1;
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIMS(vf), mask);
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIAC(vf), mask);
	} else {
		/* needs little work if we want individual irq enabled */
		IXGBE_WRITE_REG(hw, IXGBE_PVTEIMS(vf), 1 << v_idx);
	}
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_irq_trigger(struct ixgbe_adapter *adapter, u32 vf, int v_idx)
{
	struct ixgbe_hw *hw = &adapter->hw;
	u32 eics;

	DPRINTK(VIRT, INFO, "vf %u vector %d\n", vf, v_idx);

	if (v_idx < 0)
		eics = (1 << (IXGBE_MAX_VF_VECTORS)) - 1;
	else
		eics = (1 << v_idx);

	IXGBE_WRITE_REG(hw, IXGBE_PVTEICS(vf), eics);
	IXGBE_WRITE_FLUSH(hw);
}

void ixgbe_vf_mgmt(struct ixgbe_adapter *adapter) {
	/* Placeholder function */
}

/***********************NPA APIs***********************************/

/*!
 * Allocate a VF.
 *
 * This function tries to allocate a VF on a device with the requested
 * options.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param props          Requested options for the VF.
 * \param[out] vf        Identifier of the allocated VF.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND No VF was found.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_acquire(struct net_device *netdev,
			    vmk_NetVFRequirements *props,
			    vmk_VFID *vf)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	int i;
	int ret = VMK_NOT_FOUND;

	DPRINTK(VIRT, INFO, "request to allocate vf\n");

	/* validate requested options */
	if (props->version > 1) {
		DPRINTK(PROBE, INFO, "Unsupported version %08X\n",
			props->version);
		return VMK_FAILURE;
	}

	if (props->features & ~(VMK_NETVF_F_RXCSUM | VMK_NETVF_F_RXVLAN)) {
		DPRINTK(PROBE, INFO,
			"Unsupported features %04X, opt. features %04X\n",
			props->features, props->optFeatures);
		return VMK_FAILURE;
	}

	if (props->numTxQueues > 1) {
		DPRINTK(PROBE, INFO, "Unsupported TX Queues %d\n",
			props->numTxQueues);
		return VMK_FAILURE;
	}

	if (props->numRxQueues > 1) {
		DPRINTK(PROBE, INFO, "Unsupported RX Queues %d\n",
			props->numRxQueues);
		return VMK_FAILURE;
	}

	if (props->numIntrs > 3) {
		DPRINTK(PROBE, INFO, "Unsupported MSIX Vectors %d\n",
			props->numIntrs);
		return VMK_FAILURE;
	}

	if (props->mtu > 9000) {
		DPRINTK(PROBE, INFO, "Unsupported mtu %d\n",
			props->mtu);
		return VMK_FAILURE;
	}

	for (i = 0; i < adapter->num_vfs; i++) {
		if (0 == adapter->vfinfo[i].allocated) {
			adapter->vfinfo[i].allocated = 1;
			adapter->vfinfo[i].mtu = props->mtu;
			adapter->vfinfo[i].irq_rate = 8000;
			adapter->vfinfo[i].coml_method = VF_TX_COML_HEAD_WB;
			/* adapter->vfinfo[i].coml_method = VF_TX_COML_DD; */
			/* decide the buffer mode */
			if (props->mtu <= ETH_DATA_LEN)
				adapter->vfinfo[i].buffer_mode = VF_BUFFER_MODE_STANDARD;
			else
				adapter->vfinfo[i].buffer_mode = VF_BUFFER_MODE_JUMBO;
			break;
		}
	}

	if (i < adapter->num_vfs) {
		*vf = i;
		ret = VMK_OK;
		DPRINTK(VIRT, INFO, "allocated vf %u\n", *vf);
	}
	else
		DPRINTK(PROBE, INFO, "failed to allocate vf\n");
	return ret;
}

/*!
 * Initialize a VF.
 *
 * This function initializes a VF, including queues, interrupts, RSS
 * engine...
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             VF to initialize.
 * \param params         Parameters for the VF.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_BUSY      VF is already in use.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_init(struct net_device *netdev,
			 vmk_VFID vf,
			 vmk_NetVFParameters *params)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	int i;
	int errcnt = 0;
	int max_len;

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	IXGBE_CHECK_VF_ALLOC_STATE(vf);

	/*
	 * there are 3 vectors per vf, but now the plan is to
	 * use only one vector to process the rx & tx queues
	 */
	if (params->numIntrs != 1) {
		DPRINTK(PROBE, INFO, "Unsupported number of vectors (%d)\n",
			params->numIntrs);
		return VMK_BAD_PARAM;
	}

	if (params->numTxQueues != 1 || params->numRxQueues != 1) {
		DPRINTK(PROBE, INFO, "Unsupported number of queues "
			"(%d rx - %d tx)\n",
			params->numRxQueues, params->numTxQueues);
		return VMK_BAD_PARAM;
	}

	DPRINTK(VIRT, INFO, "RxQs %u TxQs %u Vectors %u\n",
		params->numRxQueues, params->numTxQueues, params->numIntrs);

	/* Check and save pointer to shared region in adapter->vfinfo struct */
	if (params->u.npa.sharedRegion != 0) {
		/* Check size requirements for shared region
		 * Currently, only care about default VLAn presence
		 * Will eventually need to account for VF stats as well
		 */
		if (params->u.npa.sharedRegionLength >=
					sizeof(struct vf_pf_shared_mem)) {
			/* Save shared memory region pointer and length */
			DPRINTK(VIRT, INFO, "Shared mem = %p,"
					"Shared mem length = %u\n",
					params->u.npa.sharedRegion,
					params->u.npa.sharedRegionLength);
			adapter->vfinfo[vf].vf_pf_shared_mem =
					(struct vf_pf_shared_mem*)
					params->u.npa.sharedRegion;
			adapter->vfinfo[vf].vf_pf_shared_mem_length =
					params->u.npa.sharedRegionLength;
			/* Initialize the shared memory structure */
			memset(adapter->vfinfo[vf].vf_pf_shared_mem,
				0x00, sizeof(struct vf_pf_shared_mem));
		} else {
			DPRINTK(PROBE, INFO, "Shared region not big enough\n");
			return VMK_BAD_PARAM;
		}
	} else {
		DPRINTK(PROBE, INFO,
				"Shared region not set up in params struct\n");
		return VMK_BAD_PARAM;
	}

	for (i = 0; i < params->numTxQueues; i++) {

		DPRINTK(VIRT, INFO, "Rx %u, addr %llx size %u len %u\n",
			i, params->u.npa.rxQueues[i].basePA,
			params->u.npa.rxQueues[i].size,
			params->u.npa.rxQueues[i].length);

		max_len = params->u.npa.rxQueues[i].size
			    * sizeof(union ixgbe_adv_rx_desc);
		if (params->u.npa.rxQueues[i].length < max_len ||
		    params->u.npa.rxQueues[i].size
		    			% IXGBE_REQ_RX_DESCRIPTOR_MULTIPLE ||
		    params->u.npa.rxQueues[i].basePA
		    			& (IXGBE_RING_BASE_ALIGNMENT-1)) {
			DPRINTK(PROBE, INFO,
				"Rx ring %d is too short or not aligned\n", i);
			errcnt++;
		}

		DPRINTK(VIRT, INFO, "Tx %u, addr %llx size %u len %u\n",
			i, params->u.npa.txQueues[i].basePA,
			params->u.npa.txQueues[i].size,
			params->u.npa.txQueues[i].length);

		/* NPA wants lots of padding for header writeback */
		max_len = (params->u.npa.txQueues[i].size
			    * sizeof(union ixgbe_adv_tx_desc))
			   + (2 * IXGBE_TX_HEAD_DMA_SPACING);
		if (params->u.npa.txQueues[i].length < max_len ||
		    params->u.npa.txQueues[i].size
					% IXGBE_REQ_TX_DESCRIPTOR_MULTIPLE ||
		    params->u.npa.txQueues[i].basePA
					& (IXGBE_RING_BASE_ALIGNMENT-1)) {
			DPRINTK(PROBE, INFO,
				"Tx ring %d is too short or not aligned\n", i);
			errcnt++;
		}

		DPRINTK(VIRT, INFO, "Vector %u, moderation level %u\n",
			i, params->u.npa.intr[i].intrMod.modLevel);
	}
	if (errcnt)
		return VMK_BAD_PARAM;

	ixgbe_vf_reset(adapter, vf);
	ixgbe_vf_configure_msix(adapter, vf, params->numTxQueues);

	/* configure rx/tx queue registers */
	for (i = 0; i < params->numTxQueues; i++) {
		ixgbe_vf_configure_rx(adapter, vf, i,
			params->u.npa.rxQueues[i].basePA,
			params->u.npa.rxQueues[i].size * sizeof(union ixgbe_adv_rx_desc),
			adapter->vfinfo[vf].mtu);
		ixgbe_vf_configure_tx(adapter, vf, i,
			params->u.npa.txQueues[i].basePA,
			params->u.npa.txQueues[i].size * sizeof(union ixgbe_adv_tx_desc));
	}

	adapter->vfinfo[vf].num_queue_pairs = params->numTxQueues;
	adapter->vfinfo[vf].init = 1;

	return VMK_OK;
}

/*!
 * Activate a VF.
 *
 * This function is called after all the initializations are done to
 * put the VF in running state (ready to send and receive traffic).
 *
 * \note This includes unblocking the VF.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF to activate.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_activate(struct net_device *netdev, vmk_VFID vf)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	/* need to read the stats because of they are read-only */
	ixgbe_vf_read_stats(adapter, vf, &adapter->vfinfo[vf].vfstats);

	/* turn on rx/tx units of the vf */
	ixgbe_vf_unblock(adapter, vf);
	ixgbe_vf_enable_queues(adapter, vf,
			       adapter->vfinfo[vf].num_queue_pairs);

	/* now vf is fully active */
	set_bit(vf, adapter->active_vfs);

	return VMK_OK;
}

/*!
 * Quiesce a VF.
 *
 * This function blocks incoming and outgoing traffic to and from a VF
 * and disable DMA and interrupts.
 *
 * \note After a call to ixgbe_vf_quiesce, the VF is not yet ready to be
 * reused and returned by ixgbe_vf_acquire.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF to quiesce.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_quiesce(struct net_device *netdev, vmk_VFID vf)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	/* remove the vf from the active list */
	clear_bit(vf, adapter->active_vfs);

	/* block vf's rx/tx units */
	ixgbe_vf_block(adapter, vf);
	ixgbe_vf_disable_queues(adapter, vf,
				adapter->vfinfo[vf].num_queue_pairs);

	return VMK_OK;
}

/*!
 * Release a VF.
 *
 * This function is called after a VF has been quiesced and
 * relinquishes the VF for future use.
 *
 * \note After a call to ixgbe_vf_release, the same VF can now be reused and
 * returned by ixgbe_vf_acquire.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF to release.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_BUSY      VF was not quiesced.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_release(struct net_device *netdev, vmk_VFID vf)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	u32 dtxswc;
	u32 regn = vf/8; /* 8 vfs per register */
	u32 lvf = vf%8;

	while (test_and_set_bit (__IXGBE_RESETTING, &adapter->state))
		usleep_range(1000, 2000);

	DPRINTK(VIRT, INFO, "vf %u\n", vf);

	IXGBE_CHECK_VF_ALLOC_STATE(vf);

	/* reset the vf */
	ixgbe_vf_reset(adapter, vf);

	/* remove the vf state from the eswitch */

	/* clear the vf's mac address */
	ixgbe_clear_vmdq(hw, adapter->vfinfo[vf].rar, vf);
	adapter->vfinfo[vf].rar = -1;

	/* remove the vf's vlan membership */
	ixgbe_vf_clear_vlvfs(adapter, vf);

	/* right now there in not much to clean in the mta
	for a given vf */

	/* restore the vmvir (default vlans) defaults */
	IXGBE_WRITE_REG(hw, IXGBE_VMVIR(vf), 0x00000000);

	/* clear the settings for the mac/vlan anti-spoofing */
	dtxswc = IXGBE_READ_REG(hw, IXGBE_DTXSWC(regn));
	dtxswc &= ~(1 << lvf);
	dtxswc &= ~(1 << (lvf + IXGBE_DTXSWC_VLAN_SPOOF_SHIFT));
	IXGBE_WRITE_REG(hw, IXGBE_DTXSWC(regn), dtxswc);

	/* cleanup vf's software state */
	memset(&(adapter->vfinfo[vf]), 0x00, sizeof(struct vf_data_storage));

	/* Clear state of reset bit */
	clear_bit(__IXGBE_RESETTING, &adapter->state);

	return VMK_OK;
}

/*!
 * Get VF information.
 *
 * This function retrieves general VF information like the addresses
 * of the BARs.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param[out] info      Information returned for the VF.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_get_info(struct net_device *netdev,
			     vmk_VFID vf, vmk_NetVFInfo *info)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_npa_plugin_data *vf_data;
	struct pci_dev *pdev;

	DPRINTK(VIRT, INFO, "vf %u\n", vf);
	IXGBE_CHECK_VF_ALLOC_STATE(vf);

	pdev = vmklnx_get_vf(netdev->pdev, vf, &info->sbdf);
	if (pdev == NULL) {
		DPRINTK(PROBE, INFO, "Cannot get pci_dev of VF %d\n", vf);
		return VMK_FAILURE;
	}

	/* get the BAR0 info. of a given VF */
	info->ptRegions[0].MA = pci_resource_start(pdev, 0);
	info->ptRegions[0].numPages = pci_resource_len(pdev, 0) / 4096;
	info->numPtRegions = 1;

	/* fill in information to be forwarded to the plugin */
	strcpy(info->u.npa.pluginName, "ixgbe_plugin-%b.so");

	vf_data = (struct ixgbe_npa_plugin_data *)info->u.npa.pluginData;
	memset(vf_data, 0x00, sizeof(struct ixgbe_npa_plugin_data));
	vf_data->buffer_mode = adapter->vfinfo[vf].buffer_mode;
	vf_data->coml_method = adapter->vfinfo[vf].coml_method;
	return VMK_OK;
}

/*!
 * Set the RSS indirection table of a VF.
 *
 * This function sets the RSS indirection table for a VF.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF to release.
 * \param table          Indirection table to set.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_set_rss_ind_table(struct net_device *netdev,
				      vmk_VFID vf, vmk_NetVFRSSIndTable *table)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);

	DPRINTK(VIRT, INFO, "vf %u\n", vf);
	IXGBE_CHECK_VF_INIT_STATE(vf);

	return VMK_FAILURE;
}
/*!
 * Get the statistics of a queue.
 *
 * This function returns statistics of a queue.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param numTxQueues
 * \param numRxQueues
 * \param[out] tqStats    Tx Statistics returned.
 * \param[out] rqStats    Rx Statistics returned.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF or queue doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_get_queue_stats(struct net_device *netdev, vmk_VFID vf,
                                    uint8_t numTxQueues, uint8_t numRxQueues,
                                    vmk_NetVFTXQueueStats *tqStats,
                                    vmk_NetVFRXQueueStats *rqStats)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct vf_stats vfstats;
	struct vf_pf_shared_mem *shared_mem = 0;
	int i;
	u64 glsopc;

	IXGBE_CHECK_VF_INIT_STATE(vf);

	shared_mem = adapter->vfinfo[vf].vf_pf_shared_mem;

	memset(tqStats, 0, numTxQueues * sizeof (vmk_NetVFTXQueueStats));
	memset(rqStats, 0, numRxQueues * sizeof (vmk_NetVFRXQueueStats));

	/* for now, no more than one rx/tx queue pair supported */
	numTxQueues = min((int)numTxQueues, 1);
	numRxQueues = min((int)numRxQueues, 1);

	ixgbe_vf_read_stats(adapter, vf, &vfstats);
	for (i = 0; i < numTxQueues; i++) {
		VF_UPDATE_STATS_32(adapter->vfinfo[vf].vfstats.gptc,
				   vfstats.gptc, tqStats[i].unicastPkts);
		VF_UPDATE_STATS_64(adapter->vfinfo[vf].vfstats.gotc,
				   vfstats.gotc, tqStats[i].unicastBytes);
		if (shared_mem) {
			vfstats.glsoc =
				shared_mem->shared_stats.LSO_count;
			vfstats.glsopc =
				shared_mem->shared_stats.LSO_pkt_count;
			vfstats.glsooc =
				shared_mem->shared_stats.LSO_byte_count;
			VF_UPDATE_STATS_64(adapter->vfinfo[vf].vfstats.glsoc,
					   vfstats.glsoc, tqStats[i].TSOPkts);
			VF_UPDATE_STATS_64(adapter->vfinfo[vf].vfstats.glsopc,
					   vfstats.glsopc, glsopc);
			VF_UPDATE_STATS_64(adapter->vfinfo[vf].vfstats.glsooc,
					   vfstats.glsooc, tqStats[i].TSOBytes);
			if (tqStats[i].unicastPkts > glsopc)
				tqStats[i].unicastPkts =
					tqStats[i].unicastPkts - glsopc;
			else
				tqStats[i].unicastPkts = 0;
			if (tqStats[i].unicastBytes > tqStats[i].TSOBytes)
				tqStats[i].unicastBytes =
					tqStats[i].unicastBytes - tqStats[i].TSOBytes;
			else
				tqStats[i].unicastBytes = 0;
		}
	}

	for (i = 0; i < numRxQueues; i++) {
		VF_UPDATE_STATS_32(adapter->vfinfo[vf].vfstats.gprc,
				   vfstats.gprc, rqStats[i].unicastPkts);
		VF_UPDATE_STATS_64(adapter->vfinfo[vf].vfstats.gorc,
				   vfstats.gorc, rqStats[i].unicastBytes);
		/* mcast & bcast is counted with the same counter */
		VF_UPDATE_STATS_32(adapter->vfinfo[vf].vfstats.mprc,
				   vfstats.mprc, rqStats[i].multicastPkts);
	}

	return VMK_OK;
}

/*!
 * Get the status of a set of queues.
 *
 * This function tells whether or not a queue has been stopped due to
 * an error.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF to release.
 * \param numTxQueues    Number of TX queues requested.
 * \param numRxQueues    Number of RX queues requested.
 * \param[out] tqStatus  Array of status structs for the TX queues.
 * \param[out] rqStatus  Array of status structs for the RX queues.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF or queue doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_get_queue_status(struct net_device *netdev, vmk_VFID vf,
				     vmk_uint8 numTxQueues,
				     vmk_uint8 numRxQueues,
				     vmk_NetVFQueueStatus *tqStatus,
				     vmk_NetVFQueueStatus *rqStatus)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	u32 rxdctl, txdctl;
	u32 i;

	DPRINTK(VIRT, INFO, "vf %u numTx %u numRx %u\n", vf, numTxQueues, numRxQueues);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	if (numTxQueues > 1 || numRxQueues > 1) {
		DPRINTK(PROBE, INFO, "Unsupported number of queues "
			"(%d rx - %d tx)\n", numRxQueues, numTxQueues);
		return VMK_BAD_PARAM;
	}

	if ((numTxQueues && tqStatus == NULL)
	    || (numRxQueues && rqStatus == NULL)) {
		DPRINTK(PROBE, INFO, "queue count but no valid status array\n");
		return VMK_BAD_PARAM;
	}

	for (i = 0; i < numTxQueues; i++) {
		txdctl = IXGBE_READ_REG(hw, IXGBE_PVFTXDCTLn(adapter, vf, i));
		tqStatus[i].stopped = ((txdctl & IXGBE_TXDCTL_ENABLE) ? false : true);
		tqStatus[i].error = 0;
	}

	for (i = 0; i < numRxQueues; i++) {
		rxdctl = IXGBE_READ_REG(hw, IXGBE_PVFRXDCTLn(adapter, vf, i));
		rqStatus[i].stopped = ((rxdctl & IXGBE_RXDCTL_ENABLE) ? false : true);
		rqStatus[i].error = 0;
	}

	return VMK_OK;
}

/*!
 * Set the unicast MAC address of a VF.
 *
 * This function sets the unicast MAC address of a VF. A VF can only
 * have exactly one unicast MAC address.
 *
 * \note A VF must have a unicast MAC address configured before
 * being unblocked. It means that this function will always be called
 * prior to unblocking the VF after the VF has been cleared or before
 * first use.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param mac            New MAC address of the VF.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_set_mac(struct net_device *netdev, vmk_VFID vf,
			    vmk_EthAddress mac)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	int rar;
	int ret = VMK_FAILURE;

	DPRINTK(VIRT, INFO, "vf %u addr %02x:%02x:%02x:%02x:%02x:%02x\n",
		vf, mac[0], mac[1], mac[2],
		mac[3], mac[4], mac[5]);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	rar = ixgbe_insert_mac_addr(hw, mac, vf);
	if (rar >= 0) {
		adapter->vfinfo[vf].rar = rar;
		ret = VMK_OK;
	}
	return ret;
}

/*!
 * Set the multicast MAC addresses to be filtered by a VF.
 *
 * This function sets a list of multicast MAC addresses for a VF.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param nmac           Number of valid entries in the mac array.
 * \param mac            Array of multicast MAC addresses of the VF.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_set_multicast(struct net_device *netdev, vmk_VFID vf,
				  size_t nmac, vmk_EthAddress *mac)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	u32 i;

	DPRINTK(VIRT, INFO, "vf %u num mcast addr %u\n", vf, nmac);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	/*
	 * rule: all the mcast addresses will be programmed in the
	 * mta to keep things simple & use rars for the ucast addresses
	 */
	for (i = 0; i < nmac; i++) {
		ixgbe_set_mta(hw, mac[i]);
	}
	ixgbe_enable_mc(hw);

	return VMK_OK;
}

/*!
 * Enable or disable receiving traffic types on a VF.
 *
 * This function sets the RX mode of a VF and indicates what traffic
 * types to accept.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param unicast        Enable or disable receiving unicast traffic.
 * \param multicast      Enable or disable receiving multicast traffic.
 * \param broadcast      Enable or disable receiving broadcast traffic.
 * \param allmulti       Enable or disable receiving all multicast traffic.
 *
 * \note Enabling receiving multicast traffic only enable traffic to
 * the MAC addresses specified by ixgbe_vf_set_multicast().
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_set_rx_mode(struct net_device *netdev, vmk_VFID vf,
				vmk_Bool unicast, vmk_Bool multicast,
				vmk_Bool broadcast, vmk_Bool allmulti)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	u32 vmolr;

	DPRINTK(VIRT, INFO,
		"vf %u ucast %u mcast %u, bcast %u, allmulti %u\n",
		vf, unicast, multicast, broadcast, allmulti);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	vmolr = IXGBE_READ_REG(hw, IXGBE_VMOLR(vf));

	if (broadcast)
		vmolr |= IXGBE_VMOLR_BAM;
	else
		vmolr &= ~IXGBE_VMOLR_BAM;

	if (multicast)
		vmolr |= IXGBE_VMOLR_ROMPE;
	else
		vmolr &= ~IXGBE_VMOLR_ROMPE;

	if (allmulti)
		vmolr |= IXGBE_VMOLR_MPE;
	else
		vmolr &= ~IXGBE_VMOLR_MPE;

	IXGBE_WRITE_REG(hw, IXGBE_VMOLR(vf), vmolr);
	IXGBE_WRITE_FLUSH(hw);

	return VMK_OK;
}

/*!
 * Enable forwarding VLAN packets within a particular range to a VF.
 *
 * This function is called when a VF becomes member of the specified
 * VLAN groups. All the packets tagged with the VLAN IDs within the
 * range must be forwarded to the specified VF.
 *
 * \note Many VFs can be member of the same VLANs group at the same
 * time.
 *
 * \note There can be many overlapping ranges per VF.
 *
 * \param netdev            Pointer to the passthrough device.
 * \param vf                Identifier of the VF.
 * \param first             Lower bound of the VLAN ID range.
 * \param last              Highest bound of the VLAN ID range.
 *
 * \note A range includes all the VLANs from first to last (included).
 *
 * \retval VMK_OK           Operation succeeded.
 * \retval VMK_NOT_FOUND    VF doesn't exist.
 * \retval VMK_NOT_READY    VF was not acquired.
 * \retval VMK_NO_RESOURCES Operation failed, out of resources.
 * \retval VMK_FAILURE      Operation failed (unknown error).
 */
static int ixgbe_vf_add_vlan_range(struct net_device *netdev, vmk_VFID vf,
				   vmk_VlanID first, vmk_VlanID last)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct vf_pf_shared_mem *shared_mem =
				adapter->vfinfo[vf].vf_pf_shared_mem;
	int i, lasti;
	int ret = VMK_OK;
	u16 vlans_enabled = 0;

	DPRINTK(VIRT, INFO, "vf %u first %u last %u\n", vf, first, last);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	/* Check to see if there is a Default VLAn already enabled
	 * and if so, fail Guest VLAN creation
	 */
	if (shared_mem->flags & VF_PF_SHARED_FLAGS_DEFAULT_VLAN_PRESENT) {
		DPRINTK(PROBE, INFO, "Default VLAN present. Returning failure\n");
		return VMK_FAILURE;
	}

	for (i = first; i <= last; i++) {
		if (0 != ixgbe_vf_set_vlvfs(adapter, vf, i, 1)) {
			DPRINTK(PROBE, INFO, "Failed to add VLAN ID %u\n", i);
			ret = VMK_NO_RESOURCES;
			lasti = i;
			break;
		}
		else {
			/* Set VLVF for the default pool as well to ensure PF/VF comm. */
			ixgbe_vf_set_vlvfs(adapter, VMDQ_P(0), i, 1);
			vlans_enabled++;
		}
	}

	/* remove the partially added list */
	if (ret != VMK_OK) {
		for (i = first; i < lasti; i++) {
			if (0 != ixgbe_vf_set_vlvfs(adapter, vf, i, 0)) {
				DPRINTK(PROBE, INFO,
					"Failed to remove VLAN ID %u\n", i);
			}
			vlans_enabled--;
		}
	}
	else {
		if (!adapter->vfinfo[vf].vlans_enabled) {
			u32 max_frame = adapter->vfinfo[vf].mtu +
					ETH_HLEN + ETH_FCS_LEN + VLAN_TAG_SIZE;
			ixgbe_vf_set_rlpml(adapter, vf, max_frame);
		}
		adapter->vfinfo[vf].vlans_enabled += vlans_enabled;
	}
	return ret;
}

/*!
 * Disable forwarding VLAN packets within a particular range to a VF.
 *
 * This function is called when a VF leaves the specified VLAN
 * groups. All the packets tagged with the VLAN IDs within the range
 * must not be forwarded anymore to the specified VF.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param first          Lower bound of the VLAN ID range.
 * \param last           Highest bound of the VLAN ID range.
 *
 * \note A range includes all the VLANs from first to last (included).
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_del_vlan_range(struct net_device *netdev, vmk_VFID vf,
				   vmk_VlanID first, vmk_VlanID last)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	int i;

	DPRINTK(VIRT, INFO, "vf %u first %u last %u\n", vf, first, last);

	IXGBE_CHECK_VF_INIT_STATE(vf);

	for (i = first; i <= last; i++) {
		if (0 != ixgbe_vf_set_vlvfs(adapter, vf, i, 0)) {
			DPRINTK(PROBE, INFO, "Failed to remove VLAN ID %u\n", i);
		} else {
			adapter->vfinfo[vf].vlans_enabled--;
			/* Check VLVF pool membership and clear VLVF if required */
			ixgbe_vf_check_or_clear_default_vlvf(adapter, i);
			if (!adapter->vfinfo[vf].vlans_enabled) {
				u32 max_frame = adapter->vfinfo[vf].mtu +
						ETH_HLEN + ETH_FCS_LEN;
				ixgbe_vf_set_rlpml(adapter, vf, max_frame);
			}
		}
	}
	return VMK_OK;
}

/*!
 * Set VLAN tagging with the specified VLAN ID and priority (NPA only).
 *
 * This function is called when the virtual switch is configured for
 * VLAN tag insertion and stripping. Every outgoing packet is tagged
 * with the VLAN ID and priority. Every incoming packet (which is in
 * this VLAN group) has its 802.1q tag stripped completely.
 *
 * \note All packets tagged with a different VLAN ID must be dropped.
 *
 * \note This function is called when the virtual switch request
 * vSwitch VLAN tagging. This mode is incompatible with guest VLAN
 * tagging, i.e none of ixgbe_vf_add_vlan_range() nor
 * ixgbe_vf_del_vlan_range() will be called when default VLAN is
 * enabled.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param vf             Identifier of the VF.
 * \param enable         Enable or disable default VLAN.
 * \param vid            VLAN group to be tagged and filtered.
 * \param prio           Priority value to be tagged.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND VF doesn't exist.
 * \retval VMK_NOT_READY VF was not acquired.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_vf_set_default_vlan(struct net_device *netdev, vmk_VFID vf,
				     vmk_Bool enable, vmk_VlanID vid,
				     vmk_VlanPriority prio)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	struct vf_pf_shared_mem *shared_mem = adapter->vfinfo[vf].vf_pf_shared_mem;
	u32 vmvir = 0;
	u32 max_frame = 0;
	int i;
	u32 reg;

	DPRINTK(VIRT, INFO, "vf %u vlan id %u enable %u prio %u\n",
		vf, vid, enable, prio);
	IXGBE_CHECK_VF_INIT_STATE(vf);

	/* Check to see if default VLAN is being enabled or changed,
	 * or if guest VLANs are already present.
	 * In this case, first clear the VLVFs and the VFTA
	 * for this vf before enabling or changing the default VLAN
	 */
	if (enable && (adapter->vfinfo[vf].vlans_enabled != 0)) {
		if (shared_mem->flags & VF_PF_SHARED_FLAGS_DEFAULT_VLAN_PRESENT) {
                       if( vid != shared_mem->npa_default_vlan_id)
                               DPRINTK(VIRT, INFO,
                                       "Changing Default VLAN for this VF\n");
                       else
                               return VMK_OK;
               } else
			DPRINTK(VIRT, INFO,
				"Removing Guest VLANs for this VF\n");
		ixgbe_vf_clear_vlvfs(adapter, vf);
		adapter->vfinfo[vf].vlans_enabled = 0;
	}

	if (!enable) {
		/* ESX- id of default VLAN not sent down while clearing it.
	 	 * So, we use the saved id in the PF's vfinfo in this case.
		 */
		if ((adapter->vfinfo[vf].vlans_enabled) &&
			(shared_mem->flags & VF_PF_SHARED_FLAGS_DEFAULT_VLAN_PRESENT)) {
			vid = shared_mem->npa_default_vlan_id;
			DPRINTK(VIRT, INFO,
				"Disabling Default VLAN ID %u\n", vid);
		} else {
			DPRINTK(VIRT, INFO, "Default VLAN NOT present\n");
			/* No vlans enabled for the VF, simply return success */
			return VMK_OK;
		}
	}

	/* add/remove vid from the vlvf */
	if (0 == ixgbe_vf_set_vlvfs(adapter, vf, vid, enable)) {
		max_frame = adapter->vfinfo[vf].mtu + ETH_HLEN + ETH_FCS_LEN;
		if (enable) {
			/* vmvir need to be set to turn on the default vlan */
			vmvir = (IXGBE_VMVIR_VLANA_DEFAULTVLAN
					<< IXGBE_VMVIR_VID_SHIFT);
			vmvir |= (vid | (prio << IXGBE_VMVIR_UP_SHIFT));
			IXGBE_WRITE_REG(hw, IXGBE_VMVIR(vf), vmvir);
			IXGBE_WRITE_FLUSH(hw);
			adapter->vfinfo[vf].vlans_enabled++;
			if (1 != adapter->vfinfo[vf].vlans_enabled)
				DPRINTK(PROBE, INFO,
					"Error: 1 default VLAN is allowed %u\n",
					adapter->vfinfo[vf].vlans_enabled);
			ixgbe_vf_set_rlpml(adapter, vf,
					   (max_frame + VLAN_TAG_SIZE));
			/* Store Default VLAN presence in shared memory area
			 * of vfinfo for VF's consumption
			 */
			shared_mem->flags |= VF_PF_SHARED_FLAGS_DEFAULT_VLAN_PRESENT;
			/* Store Default VLAN ID in vfinfo */
			shared_mem->npa_default_vlan_id = (u16)vid;
			/* Set VLVF for the default pool to allow PF/VF comm. */
			ixgbe_vf_set_vlvfs(adapter, VMDQ_P(0), vid, 1);
		} else {
			/* restore the vmvir (default vlans) defaults */
			IXGBE_WRITE_REG(hw, IXGBE_VMVIR(vf), 0x00000000);
			IXGBE_WRITE_FLUSH(hw);
			if (adapter->vfinfo[vf].vlans_enabled)
				adapter->vfinfo[vf].vlans_enabled--;
			ixgbe_vf_set_rlpml(adapter, vf, max_frame);
			if (0 != adapter->vfinfo[vf].vlans_enabled)
				DPRINTK(PROBE, INFO, "Invalid vlan count %u\n",
					adapter->vfinfo[vf].vlans_enabled);
			/* Clear Default VLAN from shared mem area in vfinfo */
			shared_mem->flags &= ~VF_PF_SHARED_FLAGS_DEFAULT_VLAN_PRESENT;
			/* Clear Default VLAN id from vfinfo */
			shared_mem->npa_default_vlan_id = 0;
			/* Check VLVF pool membership and clear VLVF if required */
			ixgbe_vf_check_or_clear_default_vlvf(adapter, vid);
		}
		return VMK_OK;
	}

	return VMK_FAILURE;
}

/*
 * Enable or disable MAC anti-spoofing for a VF (NPA only).
 *
 * This function enables or disable MAC anti-spoofing on a VF. When
 * this mode is enabled, all packets sent by the VF must have the same
 * source MAC address in the ethernet header than the address set with
 * ixgbe_vf_set_mac(). Otherwise, the packets must be dropped.
 *
 * \param netdev             Pointer to the passthrough device.
 * \param vf                 Identifier of the VF.
 * \param enable             Enable or disable dropping spoofed packets.
 *
 * \retval VMK_OK            Operation succeeded.
 * \retval VMK_NOT_FOUND     VF doesn't exist.
 * \retval VMK_NOT_READY     VF was not acquired.
 * \retval VMK_FAILURE       Operation failed (unknown error).
 *
 * \see ixgbe_vf_set_mac
 */
static int ixgbe_vf_set_antispoof(struct net_device *netdev, vmk_VFID vf,
				  vmk_Bool enable)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	struct ixgbe_hw *hw = &adapter->hw;
	u32 dtxswc;
	u32 regn = vf/8; /* 8 vfs per register */
	u32 lvf = vf%8;

	DPRINTK(VIRT, INFO, "vf %u enable %u\n", vf, enable);
	IXGBE_CHECK_VF_INIT_STATE(vf);

	dtxswc = IXGBE_READ_REG(hw, IXGBE_DTXSWC(regn));
	/* mac & vlan anti-spoofing */
	if (enable) {
		dtxswc |= (1 << lvf);
		dtxswc |= (1 << (lvf + IXGBE_DTXSWC_VLAN_SPOOF_SHIFT));
	}
	else {
		dtxswc &= ~(1 << lvf);
		dtxswc &= ~(1 << (lvf + IXGBE_DTXSWC_VLAN_SPOOF_SHIFT));
	}
	IXGBE_WRITE_REG(hw, IXGBE_DTXSWC(regn), dtxswc);
	IXGBE_WRITE_FLUSH(hw);
	return VMK_OK;
}

/*!
 * Enable forwarding packets with a particular unicast MAC address to
 * the virtual switch (NPA only).
 *
 * This function is called for every non-passthrough virtual NIC
 * connected to the virtual switch. It registers the unicast MAC
 * address of the virtual NIC. All the packets destinated to the
 * specified MAC address must be forwarded to the virtual switch.
 *
 * \note For the moment, this function only accepts unicast MAC
 * addresses and the PF must then accept all the multicast
 * traffic. This may change in future versions.
 *
 * \param netdev        Pointer to the passthrough device.
 * \param mac           MAC address to be filtered.
 *
 * \retval VMK_OK           Operation succeeded.
 * \retval VMK_NO_RESOURCES Operation failed, out of resources.
 * \retval VMK_FAILURE      Operation failed (unknown error).
 */
static int ixgbe_pf_add_mac_filter(struct net_device *netdev,
				   vmk_EthAddress mac)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);

	DPRINTK(VIRT, INFO,
		"default vf %u addr %02x:%02x:%02x:%02x:%02x:%02x\n",
		adapter->num_vfs, mac[0], mac[1], mac[2],
		mac[3], mac[4], mac[5]);

	/* set default pool's mac address */
	if (0 == ixgbe_vf_insert_addr(adapter, adapter->num_vfs, mac))
		return VMK_OK;
	else
		return VMK_NO_RESOURCES;
}

/*!
 * Disable forwarding packets with a particular unicast MAC address to
 * the virtual switch (NPA only).
 *
 * This function is called once a non-passthrough virtual NIC gets
 * disconnected. All the packets destinated to the specified MAC
 * address must no longer be forwarded to the virtual switch.
 *
 * \param netdev         Pointer to the passthrough device.
 * \param mac            MAC address to be denied.
 *
 * \retval VMK_OK        Operation succeeded.
 * \retval VMK_NOT_FOUND Address cannot be found.
 * \retval VMK_FAILURE   Operation failed (unknown error).
 */
static int ixgbe_pf_del_mac_filter(struct net_device *netdev,
				   vmk_EthAddress mac)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);

	DPRINTK(VIRT, INFO,
		"default vf %u addr %02x:%02x:%02x:%02x:%02x:%02x\n",
		adapter->num_vfs, mac[0], mac[1], mac[2],
		mac[3], mac[4], mac[5]);

	ixgbe_vf_clear_addr(adapter, adapter->num_vfs, mac);
	return VMK_OK;
}

/*!
 * Mirror all the traffic to the default queue. This is equivalent to
 * enabling or disabling promiscuous mode on a non-passthrough VM (NPA
 * only).
 *
 * This function is called when one or more non-passthrough virtual
 * NIC connected to the virtual switch enters or leaves promiscuous
 * mode. When this mode is enabled, the PF must receive all the
 * traffic from the LAN but also all the inter-VFs traffic.
 *
 * \param netdev        Pointer to the passthrough device.
 * \param enable        Enable or disable mode.
 *
 * \retval VMK_OK           Operation succeeded.
 * \retval VMK_NO_RESOURCES Operation failed, out of resources.
 * \retval VMK_FAILURE      Operation failed (unknown error).
 */
static int ixgbe_pf_mirror_all(struct net_device *netdev, vmk_Bool enable)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	DPRINTK(VIRT, INFO, "enable %u\n", enable);
	/* todo: */
	return VMK_OK;
}

/* vf api handler */
VMK_ReturnStatus ixgbe_npa_ops(struct net_device *netdev,
			       vmk_NetPTOP op, void *pargs)
{
	struct ixgbe_adapter *adapter = netdev_priv(netdev);
	VMK_ReturnStatus ret;

	/* DPRINTK(VIRT, INFO, "op code %u\n", op); */
	switch (op) {
	case VMK_NETPTOP_VF_ACQUIRE:
		{
			vmk_NetPTOPVFAcquireArgs *args = pargs;
			ret = ixgbe_vf_acquire(netdev, &args->props, &args->vf);
		}
		break;
	case VMK_NETPTOP_VF_INIT:
		{
			vmk_NetPTOPVFInitArgs *args = pargs;
			ret = ixgbe_vf_init(netdev, args->vf, &args->params);
		}
		break;
	case VMK_NETPTOP_VF_ACTIVATE:
		{
			vmk_NetPTOPVFSimpleArgs *args = pargs;
			ret = ixgbe_vf_activate(netdev, args->vf);
		}
		break;
	case VMK_NETPTOP_VF_QUIESCE:
		{
			vmk_NetPTOPVFSimpleArgs *args = pargs;
			ret = ixgbe_vf_quiesce(netdev, args->vf);
		}
		break;
	case VMK_NETPTOP_VF_RELEASE:
		{
			vmk_NetPTOPVFSimpleArgs *args = pargs;
			ret = ixgbe_vf_release(netdev, args->vf);
		}
		break;
	case VMK_NETPTOP_VF_GET_INFO:
		{
			vmk_NetPTOPVFGetInfoArgs *args = pargs;
			ret = ixgbe_vf_get_info(netdev, args->vf, &args->info);
		}
		break;
	case VMK_NETPTOP_VF_SET_RSS_IND_TABLE:
		{
			vmk_NetPTOPVFSetRSSIndTableArgs *args = pargs;
			ret = ixgbe_vf_set_rss_ind_table(netdev, args->vf,
							 &args->table);
		}
		break;
	case VMK_NETPTOP_VF_GET_QUEUE_STATS:
		{
			vmk_NetPTOPVFGetQueueStatsArgs *args = pargs;
			ret = ixgbe_vf_get_queue_stats(netdev, args->vf,
						       args->numTxQueues,
						       args->numRxQueues,
						       args->tqStats,
						       args->rqStats);
		}
		break;
	case VMK_NETPTOP_VF_GET_QUEUE_STATUS:
		{
			vmk_NetPTOPVFGetQueueStatusArgs *args = pargs;
			ret = ixgbe_vf_get_queue_status(netdev, args->vf,
							args->numTxQueues,
							args->numRxQueues,
							args->tqStatus,
							args->rqStatus);
		}
		break;
	case VMK_NETPTOP_VF_SET_MAC:
		{
			vmk_NetPTOPVFSetMacArgs *args = pargs;
			ret = ixgbe_vf_set_mac(netdev, args->vf, args->mac);
		}
		break;
	case VMK_NETPTOP_VF_SET_MULTICAST:
		{
			vmk_NetPTOPVFSetMulticastArgs *args = pargs;
			ret = ixgbe_vf_set_multicast(netdev, args->vf,
						     args->nmac,
						     args->macList);
		}
		break;
	case VMK_NETPTOP_VF_SET_RX_MODE:
		{
			vmk_NetPTOPVFSetRxModeArgs *args = pargs;
			ret = ixgbe_vf_set_rx_mode(netdev, args->vf,
						   args->unicast,
						   args->multicast,
						   args->broadcast,
						   args->allmulti);
		}
		break;
	case VMK_NETPTOP_VF_ADD_VLAN_RANGE:
		{
			vmk_NetPTOPVFVlanRangeArgs *args = pargs;
			ret = ixgbe_vf_add_vlan_range(netdev, args->vf,
						      args->first,
						      args->last);
		}
		break;
	case VMK_NETPTOP_VF_DEL_VLAN_RANGE:
		{
			vmk_NetPTOPVFVlanRangeArgs *args = pargs;
			ret = ixgbe_vf_del_vlan_range(netdev, args->vf,
						      args->first,
						      args->last);
		}
		break;
	case VMK_NETPTOP_VF_SET_DEFAULT_VLAN:
		{
			vmk_NetPTOPVFSetDefaultVlanArgs *args = pargs;
			ret = ixgbe_vf_set_default_vlan(netdev, args->vf,
							args->enable,
							args->vid,
							args->prio);
		}
		break;
	case VMK_NETPTOP_VF_SET_ANTISPOOF:
		{
			vmk_NetPTOPVFSetAntispoofArgs *args = pargs;
			ret = ixgbe_vf_set_antispoof(netdev, args->vf,
						     args->enable);
		}
		break;
	case VMK_NETPTOP_PF_ADD_MAC_FILTER:
		{
			vmk_NetPTOPPFMacFilterArgs *args = pargs;
			ret = ixgbe_pf_add_mac_filter(netdev, args->mac);
		}
		break;
	case VMK_NETPTOP_PF_DEL_MAC_FILTER:
		{
			vmk_NetPTOPPFMacFilterArgs *args = pargs;
			ret = ixgbe_pf_del_mac_filter(netdev, args->mac);
		}
		break;
	case VMK_NETPTOP_PF_MIRROR_ALL:
		{
			vmk_NetPTOPPFMirrorAllArgs *args = pargs;
			ret = ixgbe_pf_mirror_all(netdev, args->enable);
		}
		break;
	default:
		DPRINTK(PROBE, INFO, "Unhandled op %u\n", op);
		ret = VMK_FAILURE;
		break;
	}

	return ret;
}

