/*******************************************************************************

  Intel 10 Gigabit PCI Express Linux driver
  Copyright(c) 1999 - 2011 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  e1000-devel Mailing List <e1000-devel@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#error this file should be compiled only when IXGBE_ESX_NPA is defined

#ifndef _IXGBE_NPA_H_
#define _IXGBE_NPA_H_


#define IXGBE_MAX_HDR_SIZE 960
#define IXGBE_MAX_VF_VECTORS 3

#define IXGBE_SRRCTL_BSIZEHDRSIZE_SHIFT	2

/* 802.1q VLAN Packet Size */
#define VLAN_TAG_SIZE                 4    /* 802.3ac tag (not DMA'd) */

/* following defines should move to the right header file */

#define IXGBE_VLVF_VLANID_MASK        0x00000FFF

#define IXGBE_VMVIR_VLANA_DEFAULTVLAN 1
#define IXGBE_VMVIR_VID_SHIFT         30
#define IXGBE_VMVIR_UP_SHIFT          12

#define IXGBE_DTXSWC_VLAN_SPOOF_SHIFT 8
#define IXGBE_DTXSWC_MAC_SPOOF_MASK   0x000000FF /* Per VF MAC spoof control */
#define IXGBE_DTXSWC_VLAN_SPOOF_MASK  0x0000FF00 /* Per VF VLAN spoof control */


/* add the delta of the 64-bit stats to the counter */
/* all the variables are 64-bit */
#define VF_UPDATE_STATS_64(last_counter, current_counter, counter)	\
	{						\
		if (current_counter < last_counter) { \
			counter += (0xFFFFFFFFFFFFFFFFLL - last_counter); \
			counter += current_counter; \
		} else { \
			counter += (current_counter - last_counter); \
		} \
		last_counter = current_counter; \
	}

/* add the delta of the 32-bit stats to the counter */
/* last_counter/current_counter are 32-bit & counter is 64-bit */
#define VF_UPDATE_STATS_32(last_counter, current_counter, counter)	\
	{						\
		if (current_counter < last_counter) { \
			counter += (0xFFFFFFFF - last_counter); \
			counter += current_counter; \
		} else { \
			counter += (current_counter - last_counter); \
		} \
		last_counter = current_counter; \
	}

/* Translated register #defines */
#define IXGBE_PVFCTRL(P)       (0x00300 + (4 * P))
#define IXGBE_PVFSTATUS(P)     (0x00008 + (0 * P))
#define IXGBE_PVFLINKS(P)      (0x042A4 + (0 * P))
#define IXGBE_PVFRTIMER(P)     (0x00048 + (0 * P))
#define IXGBE_PVFMAILBOX(P)    (0x04C00 + (4 * P))
#define IXGBE_PVFRXMEMWRAP(P)  (0x03190 + (0 * P))
#define IXGBE_PVTEICR(P)       (0x00B00 + (4 * P))
#define IXGBE_PVTEICS(P)       (0x00C00 + (4 * P))
#define IXGBE_PVTEIMS(P)       (0x00D00 + (4 * P))
#define IXGBE_PVTEIMC(P)       (0x00E00 + (4 * P))
#define IXGBE_PVTEIAC(P)       (0x00F00 + (4 * P))
#define IXGBE_PVTEIAM(P)       (0x04D00 + (4 * P))
#define IXGBE_PVTEITR(P)       (((P) < 24) ? (0x00820 + ((P) * 4)) : \
                                             (0x012300 + (((P) - 24) * 4)))
#define IXGBE_PVTIVAR(P)       (0x12500 + (4 * P))
#define IXGBE_PVTIVAR_MISC(P)  (0x04E00 + (4 * P))
#define IXGBE_PVTRSCINT(P)     (0x12000 + (4 * P))
/* #define IXGBE_VFPBACL  still says TBD in EAS */
#define IXGBE_PVFRDBAL(P)      ((P < 64) ? (0x01000 + (0x40 * P)) : (0x0D000 + (0x40 * (P - 64))))
#define IXGBE_PVFRDBAH(P)      ((P < 64) ? (0x01004 + (0x40 * P)) : (0x0D004 + (0x40 * (P - 64))))
#define IXGBE_PVFRDLEN(P)      ((P < 64) ? (0x01008 + (0x40 * P)) : (0x0D008 + (0x40 * (P - 64))))
#define IXGBE_PVFRDH(P)        ((P < 64) ? (0x01010 + (0x40 * P)) : (0x0D010 + (0x40 * (P - 64))))
#define IXGBE_PVFRDT(P)        ((P < 64) ? (0x01018 + (0x40 * P)) : (0x0D018 + (0x40 * (P - 64))))
#define IXGBE_PVFRXDCTL(P)     ((P < 64) ? (0x01028 + (0x40 * P)) : (0x0D028 + (0x40 * (P - 64))))
#define IXGBE_PVFSRRCTL(P)     ((P < 64) ? (0x01014 + (0x40 * P)) : (0x0D014 + (0x40 * (P - 64))))
#define IXGBE_PVFPSRTYPE(P)    (0x0EA00 + (4 * P))
#define IXGBE_PVFTDBAL(P)      (0x06000 + (0x40 * P))
#define IXGBE_PVFTDBAH(P)      (0x06004 + (0x40 * P))
#define IXGBE_PVFTTDLEN(P)     (0x06008 + (0x40 * P))
#define IXGBE_PVFTDH(P)        (0x06010 + (0x40 * P))
#define IXGBE_PVFTDT(P)        (0x06018 + (0x40 * P))
#define IXGBE_PVFTXDCTL(P)     (0x06028 + (0x40 * P))
#define IXGBE_PVFTDWBAL(P)     (0x06038 + (0x40 * P))
#define IXGBE_PVFTDWBAH(P)     (0x0603C + (0x40 * P))
#define IXGBE_PVFDCA_RXCTRL(P) ((P < 64) ? (0x0100C + (0x40 * P)) : (0x0D00C + (0x40 * (P - 64))))
#define IXGBE_PVFDCA_TXCTRL(P) (0x0600C + (0x40 * P))
#define IXGBE_PVFGPRC(x)       (0x0101C + (0x40 * x))
#define IXGBE_PVFGPTC(x)       (0x08300 + (0x04 * x))
#define IXGBE_PVFGORC_LSB(x)   (0x01020 + (0x40 * x))
#define IXGBE_PVFGORC_MSB(x)   (0x0D020 + (0x40 * x))
#define IXGBE_PVFGOTC_LSB(x)   (0x08400 + (0x08 * x))
#define IXGBE_PVFGOTC_MSB(x)   (0x08404 + (0x08 * x))
#define IXGBE_PVFGMPRC(x)      (0x0D01C + (0x40 * x))


/* irq */
/* the following define is wrong in the ixgbe_vf.h */
#define IXGBE_VFLRE(_i)        (0x00700 + ((_i) * 4))
/* extra T in the name */
#define IXGBE_PVFTDLEN(P)      (0x06008 + (0x40 * P))
#define IXGBE_VMVIR(P)         (0x08000 + (4 * P))
#define IXGBE_DTXSWC(P)        (0x08200 + (4 * P))

/* assumption: operating mode, either 64 or 32 pools */
#define IXGBE_PVTEITRn(a, vf, n)       (IXGBE_PVTEITR(127 - 2*vf + n))

#define IXGBE_PVTIVARn(a, vf, n)       ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVTIVAR(2*vf + n)   : IXGBE_PVTIVAR(vf))
/* tx */
#define IXGBE_PVFTDBALn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDBAL(4*vf + n)  : IXGBE_PVFTDBAL(2*vf + n))
#define IXGBE_PVFTDBAHn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDBAH(4*vf + n)  : IXGBE_PVFTDBAH(2*vf + n))
#define IXGBE_PVFTDLENn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDLEN(4*vf + n)  : IXGBE_PVFTDLEN(2*vf + n))
#define IXGBE_PVFTDHn(a, vf, n)        ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDH(4*vf + n)    : IXGBE_PVFTDH(2*vf + n))
#define IXGBE_PVFTDTn(a, vf, n)        ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDT(4*vf + n)    : IXGBE_PVFTDT(2*vf + n))
#define IXGBE_PVFTXDCTLn(a, vf, n)     ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTXDCTL(4*vf + n) : IXGBE_PVFTXDCTL(2*vf + n))
#define IXGBE_PVFTDWBALn(a, vf, n)     ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDWBAL(4*vf + n) : IXGBE_PVFTDWBAL(2*vf + n))
#define IXGBE_PVFTDWBAHn(a, vf, n)     ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFTDWBAH(4*vf + n) : IXGBE_PVFTDWBAH(2*vf + n))
#define IXGBE_PVFDCA_TXCTRLn(a, vf, n) ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFDCA_TXCTRL(4*vf + n) : IXGBE_PVFDCA_TXCTRL(2*vf + n))
/* rx */
#define IXGBE_PVFRDBALn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRDBAL(4*vf + n)  : IXGBE_PVFRDBAL(2*vf + n))
#define IXGBE_PVFRDBAHn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRDBAH(4*vf + n)  : IXGBE_PVFRDBAH(2*vf + n))
#define IXGBE_PVFRDLENn(a, vf, n)      ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRDLEN(4*vf + n)  : IXGBE_PVFRDLEN(2*vf + n))
#define IXGBE_PVFRDHn(a, vf, n)        ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRDH(4*vf + n)    : IXGBE_PVFRDH(2*vf + n))
#define IXGBE_PVFRDTn(a, vf, n)        ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRDT(4*vf + n)    : IXGBE_PVFRDT(2*vf + n))
#define IXGBE_PVFRXDCTLn(a, vf, n)     ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFRXDCTL(4*vf + n) : IXGBE_PVFRXDCTL(2*vf + n))
#define IXGBE_PVFSRRCTLn(a, vf, n)     ((a->flags & IXGBE_FLAG_DCB_ENABLED) ?  IXGBE_PVFSRRCTL(4*vf + n) : IXGBE_PVFSRRCTL(2*vf + n))

#define VF_TX_COML_HEAD_WB              1
#define VF_TX_COML_DD                   2

#define VF_BUFFER_MODE_STANDARD	        1
#define VF_BUFFER_MODE_JUMBO            2

#define VF_DEFAULT_VLAN_PRESENT         1
#define VF_DEFAULT_VLAN_CLEARED         0

/* amount of space between ring and head DMA address */
#define IXGBE_TX_HEAD_DMA_SPACING      128

/* alignment of ring base in bytes */
#define IXGBE_RING_BASE_ALIGNMENT      128

/* info. to be passed on to the plugin from PF */
struct ixgbe_npa_plugin_data {
	u16 device_id;
	u16 subsystem_vendor_id;
	u16 subsystem_device_id;
	u16 vendor_id;

	u8 buffer_mode;
	u8 coml_method;
};

/* eswitch control apis */
s32 ixgbe_npa_initialize(struct ixgbe_adapter *adapter);
void ixgbe_npa_cleanup(struct ixgbe_adapter *adapter);

/* vf control apis */
void ixgbe_vf_unblock(struct ixgbe_adapter *adapter, u32 vf);
void ixgbe_vf_block(struct ixgbe_adapter *adapter, u32 vf);
void ixgbe_vf_reset(struct ixgbe_adapter *adapter, u32 vf);

/* vf configuration apis */
void ixgbe_vf_set_rah_pool(struct ixgbe_adapter *adapter, u32 vf, u32 entry);
void ixgbe_vf_set_rlpml(struct ixgbe_adapter *adapter, u32 vf, u32 size);
void ixgbe_vf_configure_tx(struct ixgbe_adapter *adapter, u32 vf,
							u32 queue_index, u64 tdba, u32 tdlen);
void ixgbe_vf_configure_rx(struct ixgbe_adapter *adapter, u32 vf,
							u32 queue_index,
							u64 rdba, u32 rdlen,
							u32 mtu);
void ixgbe_vf_configure_srrctl(struct ixgbe_adapter *adapter, u32 vf,
								u32 queue_index, u32 mtu);
void ixgbe_vf_enable_queues(struct ixgbe_adapter *adapter, u32 vf,
			    u16 num_queue_pairs);
void ixgbe_vf_disable_queues(struct ixgbe_adapter *adapter, u32 vf,
			     u16 num_queue_pairs);


/* mac addr filter apis */
s32 ixgbe_vf_insert_addr(struct ixgbe_adapter *adapter, u32 vf, u8 *addr);
void ixgbe_vf_clear_addr(struct ixgbe_adapter *adapter, u32 vf, u8 *addr);

/* vlan filter apis */
void ixgbe_vf_clear_vlvfs(struct ixgbe_adapter *adapter, u32 vf);
s32  ixgbe_vf_set_vlvfs(struct ixgbe_adapter *adapter, u32 vf, u32 vid, bool add);
void ixgbe_vf_check_or_clear_default_vlvf(struct ixgbe_adapter *adapter, u32 vlan);

/* vf irq control apis */
void ixgbe_vf_irq_trigger(struct ixgbe_adapter *adapter, u32 vf, int v_idx);
void ixgbe_vf_irq_enable(struct ixgbe_adapter *adapter, u32 vf, int v_idx);
void ixgbe_vf_irq_disable(struct ixgbe_adapter *adapter, u32 vf, int v_idx);

/* vf stats api */
void ixgbe_vf_read_stats(struct ixgbe_adapter *adapter, int vf, struct vf_stats *pvf_stats);

/* npa sriov */
int ixgbe_npa_enable_sriov(struct ixgbe_adapter *adapter);
void ixgbe_npa_disable_sriov(struct ixgbe_adapter *adapter);

/* print vf registers */
void ixgbe_vf_print_regs(struct ixgbe_adapter *adapter, u32 vf);
void ixgbe_vf_print_eswitch_regs(struct ixgbe_adapter *adapter);

/* vf management api */
void ixgbe_vf_mgmt(struct ixgbe_adapter *adapter);

/* npa ops handler */
VMK_ReturnStatus ixgbe_npa_ops(struct net_device *netdev, vmk_NetPTOP op, void *args);
#endif /* _IXGBE_NPA_H_ */
