/*********************************************************************/
/*                     Intel Confidential                            */
/*********************************************************************/

/**********************************************************************
**                                                                   **
**                I N T E L   P R O P R I E T A R Y                  **
**                                                                   **
**   COPYRIGHT (c)  2001 - 2011  BY  INTEL  CORPORATION.       ALL   **
**   RIGHTS RESERVED.   NO PART OF THIS PROGRAM OR PUBLICATION MAY   **
**   BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A   **
**   RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER   **
**   LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,   **
**   MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT   **
**   THE PRIOR WRITTEN PERMISSION OF :                               **
**                                                                   **
**                      INTEL  CORPORATION                           **
**                                                                   **
**                2200 MISSION COLLEGE BOULEVARD                     **
**                                                                   **
**             SANTA  CLARA,  CALIFORNIA  95052-8119                 **
**                                                                   **
**********************************************************************/

/**********************************************************************
*                                                                     *
* INTEL CORPORATION PROPRIETARY INFORMATION                           *
* This software is supplied under the terms of a license agreement    *
* with Intel Corporation and may not be copied nor disclosed          *
* except in accordance with the terms of that agreement.              *
*                                                                     *
* Module Name:                                                        *
*   linuxdriver.h                                                     *
*                                                                     *
* Abstract:                                                           *
*   This contains the macros and functions declarations necessary for *
*   the Linux Driver component of NAL to function as a driver.        *
*                                                                     *
***********************************************************************
* VSS Revision Control Information:
* ---------------------------------
*   $Workfile: linuxdriver.c $
*   $Date: 2011/06/20 13:26:27 $
*   $Archive: /QV2.0/nal/src/linux/driver/linuxnaldriver.c $
*   $Revision: 1.3 $
**********************************************************************/

#ifndef _LINUXDRIVER_H_
#define _LINUXDRIVER_H_

#include "linuxdriverversion.h"

#define NAL_LINUX_DRIVER_FALSE               0
#define NAL_LINUX_DRIVER_TRUE                1

#define NAL_LINUX_DRIVER_SUCCESS             0
#define NAL_LINUX_DRIVER_FAILED              1
#define NAL_LINUX_DRIVER_INVALID_PARAMETER   2
#define NAL_LINUX_DRIVER_GET_STATUS(Status) \
    (Status == 0) ? (NAL_LINUX_DRIVER_SUCCESS) : (NAL_LINUX_DRIVER_FAILED)

/* Macros for domain kernel privileges */
#define NAL_LINUX_DRIVER_OS_DOMAIN_BAREMETAL 0
#define NAL_LINUX_DRIVER_OS_DOMAIN_0         1
#define NAL_LINUX_DRIVER_OS_DOMAIN_U         2

/* This should be updated whenever there is a change in linux driver */
/*#define LINUX_DRIVER_VERSION      "1.1.4.0"*/

/* NAL linux driver version defines */
#define DRIVER_MAJOR_VERSION   1
#define DRIVER_MINOR_VERSION   2
#define DRIVER_BUILD_NUMBER    0
#define DRIVER_FIX_NUMBER      0

#ifndef MINOR_VERSION
#define MINOR_VERSION          DRIVER_MINOR_VERSION
#endif

/* Version macros */
#if MINOR_VERSION < 10
#define _MAKE_VER_STRING(maj, min, build, fix)    #maj ".0" #min "." #build "." #fix
#else
#define _MAKE_VER_STRING(maj, min, build, fix)    #maj "." #min "." #build "." #fix
#endif
#define MAKE_VER_STRING(maj, min, submin, bld)    _MAKE_VER_STRING(maj, min, submin, bld)

#define NAL_LINUX_DRIVER_NAME     "nal"  /* Also defined in linuxdevice.h for user mode. */
#define DRIVER_COMPANYNAME        "Intel Corporation "
#define DRIVER_COPYRIGHT_YEARS    "2002-2011 "
#define DRIVER_FILEDESCRIPTION    "Intel(R) Network Adapter Diagnostic Driver "
#define DRIVER_LEGALCOPYRIGHT     "Copyright (C) " DRIVER_COPYRIGHT_YEARS DRIVER_COMPANYNAME "All Rights Reserved."

/* NAL driver version info strings */
#define DRIVER_FILEVERSION_STR    MAKE_VER_STRING(DRIVER_MAJOR_VERSION, DRIVER_MINOR_VERSION, DRIVER_BUILD_NUMBER, DRIVER_FIX_NUMBER)
#define DRIVER_DESCRIPTION DRIVER_FILEDESCRIPTION DRIVER_FILEVERSION_STR


void
nal_driver_get_version(
    char* Version);

void
nal_spin_lock(void);

void
nal_spin_unlock(void);

unsigned int
nal_get_os_run_domain(void);

#endif /* _LINUXDRIVER_H_ */
