/*********************************************************************/
/*                     Intel Confidential                            */
/*********************************************************************/

/**********************************************************************
**                                                                   **
**                I N T E L   P R O P R I E T A R Y                  **
**                                                                   **
**   COPYRIGHT (c)  2001 - 2011  BY  INTEL  CORPORATION.       ALL   **
**   RIGHTS RESERVED.   NO PART OF THIS PROGRAM OR PUBLICATION MAY   **
**   BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A   **
**   RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER   **
**   LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,   **
**   MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT   **
**   THE PRIOR WRITTEN PERMISSION OF :                               **
**                                                                   **
**                      INTEL  CORPORATION                           **
**                                                                   **
**                2200 MISSION COLLEGE BOULEVARD                     **
**                                                                   **
**             SANTA  CLARA,  CALIFORNIA  95052-8119                 **
**                                                                   **
**********************************************************************/

/**********************************************************************
*                                                                     *
* INTEL CORPORATION PROPRIETARY INFORMATION                           *
* This software is supplied under the terms of a license agreement    *
* with Intel Corporation and may not be copied nor disclosed          *
* except in accordance with the terms of that agreement.              *
*                                                                     *
* Module Name:                                                        *
*   linuxdriveros_i.c                                                 *
*                                                                     *
* Abstract:                                                           *
*   This is the driver OSI portion of NAL Linux.  These same          *
*   function implementations will appear in Linuxlibrary.c. This file *
*   provides the necessary means to call the IOCTL needed and returns *
*   the correct values for the OS Interface of the NAL for Linux.     *
*                                                                     *
***********************************************************************
* VSS Revision Control Information:
* ---------------------------------
*   $Workfile: linuxdriveros_i.c $
*   $Date: 2011/06/20 13:26:27 $
*   $Archive: /Quartzville2.0/nal/src/linux/driver/linuxdriveros_i.c $
*   $Revision: 1.20 $
**********************************************************************/

#include <asm/io.h>             /* for I/O routines, memory mapping. */
#include <asm/mman.h>           /* for memory access defines */
#include <linux/slab.h>         /* Allocation of memory */
#include <asm/atomic.h>         /* Atomic operations */
#include <linux/sched.h>        /* Interrupt routines */
#include <asm/page.h>           /* Page size definition */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/pci_ids.h>
#include <linux/errno.h>
#include <linux/fs.h>           /* Memory mapping functions */
#include <linux/mm.h>           /* Memory mapping functions */
#include <asm/uaccess.h>
#include <linux/pci.h>
#include <linux/delay.h>

#include "linuxdriver.h"
#include "linuxdriveros_i.h"


#if 0
unsigned char            Global_DebugPrintEnabled = NAL_LINUX_DRIVER_TRUE;
#else
unsigned char            Global_DebugPrintEnabled = NAL_LINUX_DRIVER_FALSE;
#endif

/***************************************************************************
**
** Name:            nal_read_port8()
**
** Description:     Reads an 8 bit value from a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port = the I/O port from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned char
nal_read_port8(
    unsigned short Port
    )
{
    return inb(Port);
}

/***************************************************************************
**
** Name:            nal_read_port16()
**
** Description:     Reads a 16bit value from a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port = the I/O port from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned short
nal_read_port16(
    unsigned short Port
    )
{
    return inw(Port);
}

/***************************************************************************
**
** Name:            nal_read_port32()
**
** Description:     Reads a 32bit value from a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port = the I/O port from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned int
nal_read_port32(
    unsigned short Port
    )
{
    return inl(Port);
}

/***************************************************************************
**
** Name:            nal_write_port8()
**
** Description:     Writes an 8bit value to a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port  = the I/O port to which to write.
**                  Value = the value to write.
**
** Returns:         TRUE
**
****************************************************************************/
unsigned char
nal_write_port8(
    unsigned short   Port,
    unsigned char    Value
    )
{
    outb(Value, Port);
    return NAL_LINUX_DRIVER_TRUE;
}

/***************************************************************************
**
** Name:            nal_write_port16()
**
** Description:     Writes a 16bit value to a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port  = the I/O port to which to write.
**                  Value = the value to write.
**
** Returns:         TRUE
**
****************************************************************************/
unsigned char
nal_write_port16(
    unsigned short   Port,
    unsigned short   Value
    )
{
    outw(Value, Port);
    return NAL_LINUX_DRIVER_TRUE;
}

/***************************************************************************
**
** Name:            nal_write_port32()
**
** Description:     Writes a 32 bit value to a specified I/O Port
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Port  = the I/O port to which to write.
**                  Value = the value to write.
**
** Returns:         TRUE
**
****************************************************************************/
unsigned char
nal_write_port32(
    unsigned short   Port,
    unsigned int     Value
    )
{
    outl(Value, Port);
    return NAL_LINUX_DRIVER_TRUE;
}

/***************************************************************************
**
** Name:            nal_read_register8()
**
** Description:     Reads an 8 bit value from the specified memory address
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = the memory address from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned char
nal_read_register8(
    void*   Address
    )
{
    unsigned char  Value = 0;

    if(Address != NULL)
    {
        Value = readb((char*)Address);
    }
    return Value;
}

/***************************************************************************
**
** Name:            nal_read_register16()
**
** Description:     Reads a 16 bit value from the specified memory address
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = the memory address from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned short
nal_read_register16(
    void*    Address
    )
{
    unsigned short  Value = 0;

    if(Address != NULL)
    {
        Value = readw((char*)Address);
    }
    return Value;
}

/***************************************************************************
**
** Name:            nal_read_register32()
**
** Description:     Reads a 32bit value from the specified memory address
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = the memory address from which to read.
**
** Returns:         The data read.
**
****************************************************************************/
unsigned int
nal_read_register32(
    void*        Address
    )
{
    unsigned int  Value = 0;

    if(Address != NULL)
    {
        Value = readl((char*)Address);
    }
    return Value;
}

/***************************************************************************
**
** Name:            nal_write_register8()
**
** Description:     Writes an 8bit value to a specified Memory Address.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = Pointer to the address to write to.
**                  Value    = the value to write.
**
** Returns:         TRUE if written, FALSE otherwise.
**
****************************************************************************/
unsigned char
nal_write_register8(
    void*                   Address,
    unsigned char           Value
    )
{
    unsigned char Success = NAL_LINUX_DRIVER_FALSE;

    if(Address != NULL)
    {
        writeb(Value, (char*)Address);
        Success = NAL_LINUX_DRIVER_TRUE;
    }
    return Success;
}

/***************************************************************************
**
** Name:            nal_write_register16()
**
** Description:     Writes a 16bit value to a specified Memory Address.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = Pointer to the address to write to.
**                  Value    = the value to write.
**
** Returns:         TRUE if written, FALSE otherwise.
**
****************************************************************************/
unsigned char
nal_write_register16(
    void*                Address,
    unsigned short       Value
    )
{
    unsigned char Success = NAL_LINUX_DRIVER_FALSE;

    if(Address != NULL)
    {
        writew(Value, (char*)Address);
        Success = NAL_LINUX_DRIVER_TRUE;
    }
    return Success;
}

/***************************************************************************
**
** Name:            nal_write_register32()
**
** Description:     Writes a 32bit value to a specified Memory Address.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = Pointer to the address to write to.
**                  Value    = the value to write.
**
** Returns:         TRUE if written, FALSE otherwise.
**
****************************************************************************/
unsigned char
nal_write_register32(
    void*                   Address,
    unsigned int            Value
    )
{
    unsigned char Success = NAL_LINUX_DRIVER_FALSE;

    if(Address != NULL)
    {
        writel(Value, (char*)Address);
        Success = NAL_LINUX_DRIVER_TRUE;
    }
    return Success;
}

/***************************************************************************
**
** Name:            nal_mmap_address()
**
** Description:     Maps a physical address to a virtual address for I/O.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       VirtualAddress  = Pointer to the Virtual Address to map to
**                  PhysicalAddress = The address to map
**                  Length          = Number of bytes to map
**
** Returns:         on success: NAL_LINUX_DRIVER_SUCCESS
**                  on failure: NAL_LINUX_DRIVER_INVALID_PARAMETER
**                              NAL_LINUX_DRIVER_FAILED
**                  Length contains the number of bytes actually mapped.
**                  VirtualAddress contains the base address of the mapped
**                  region.
**
****************************************************************************/
unsigned int
nal_mmap_address(
    void**                VirtualAddress,
    unsigned long long    PhysicalAddress,
    unsigned int*         Length
    )
{
    unsigned int Status = NAL_LINUX_DRIVER_INVALID_PARAMETER;

    if(VirtualAddress != NULL && Length != NULL)
    {
        *VirtualAddress = ioremap(PhysicalAddress, (unsigned long long)(*Length));
        if(*VirtualAddress != NULL)
        {
            Status = NAL_LINUX_DRIVER_SUCCESS;
        }
        else
        {
            Status = NAL_LINUX_DRIVER_FAILED;
        }
    }
    return Status;
}

/***************************************************************************
**
** Name:            nal_unmap_address()
**
** Description:     Unmaps an address mapped with nal_mmap_address().
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       VirtualAddress  = Pointer to the Virtual Address to map to
**                  PhysicalAddress = The physical address mapped to the
**                                    VirtualAddress parameter.
**                  Length          = Number of bytes mapped.
**
** Returns:         on success: NAL_LINUX_DRIVER_SUCCESS
**                  on failure: NAL_LINUX_DRIVER_INVALID_PARAMETER
**
****************************************************************************/
unsigned int
nal_unmap_address(
    void*                 VirtualAddress,
    unsigned long long    PhysicalAddress,
    unsigned int          Length
    )
{
    unsigned int Status = NAL_LINUX_DRIVER_INVALID_PARAMETER;
    if(VirtualAddress != NULL)
    {
        iounmap(VirtualAddress);
        Status = NAL_LINUX_DRIVER_SUCCESS;
    }
    return Status;
}

/***************************************************************************
**
** Name:            nal_delay_milliseconds()
**
** Description:     Stops the thread for a specified amt of milliseconds
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Milliseconds = Number of milliseconds to pause.
**
** Returns:         None.
**
****************************************************************************/
void
nal_delay_milliseconds(
    unsigned int Milliseconds
    )
{
    mdelay(Milliseconds);
}

/***************************************************************************
**
** Name:            nal_delay_microseconds()
**
** Description:     Stops the thread for a specified amt of microseconds
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Microseconds = Number of microseconds to pause.
**
** Returns:         None.
**
****************************************************************************/
void
nal_delay_microseconds(
    unsigned int Microseconds
    )
{
    /* If more than 1000, use nal_delay_milliseconds */
    if(Microseconds>1000)
    {
        nal_delay_milliseconds((unsigned int)(Microseconds/1000));
        Microseconds %= 1000;
    }
    udelay(Microseconds);
}

/***************************************************************************
**
** Name:            nal_get_time_stamp()
**
** Description:     Returns the TSC (Time Stamp Counter - IA32) or the ITC
**                  (Interval Time Counter - IA64).
**
** Author:          MJP
**
** Born on Date:    09/10/2001
**
** Arguments:       None
**
** Returns:         64-bit time stamp that represents the number of clock
**                  increments since the CPU has started.
**
****************************************************************************/
unsigned long long
nal_get_time_stamp(void)
{
    unsigned int  Low  = 0;
    unsigned int  High = 0;

   // rdtsc(Low, High);
    return (unsigned long long)((((unsigned long long)High) << 32) | Low);
}

/***************************************************************************
**
** Name:            nal_get_time_stamps_per_microsecond()
**
** Description:     Returns the number of time stamps in each microsecond.
**
** Author:          MJP
**
** Born on Date:    01/30/2002
**
** Arguments:       None
**
** Returns:         The number of time stamps that will occur during
**                  a microsecond.
**
****************************************************************************/
unsigned long long
nal_get_time_stamps_per_microsecond(void)
{
    return (unsigned long long)0;
}

/***************************************************************************
**
** Name:            nal_allocate_memory()
**
** Description:     Allocates the specified number of pagable bytes.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       ByteCount  = Number of bytes to allocate.
**                  Flags      = Flags controlling behavior of kmalloc
**
** Returns:         Pointer to the memory buffer, NULL if failed.
**
** Note:            This is used ONLY to allocate kernel memory from functions
**                  within the kernel portion of the NAL. No IOCTL call will
**                  resolve to this. Instead, they will use malloc.
****************************************************************************/
void*
nal_allocate_memory(
    unsigned int      ByteCount,
    int               Flags
    )
{
    void*   Address = NULL;
    int     KmallocFlags = 0;

    NAL_LINUX_DRIVER_SET_KMALLOC_FLAGS(Flags, KmallocFlags);
    Address = kmalloc(ByteCount, KmallocFlags); //vmalloc(ByteCount);
    if(Address != NULL)
    {
        memset(Address, 0, ByteCount);
    }
    return Address;
}

/***************************************************************************
**
** Name:            nal_free_memory()
**
** Description:     Frees the memory allocated at the buffer address.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = Pointer to the address to free.
**
** Returns:         Nothing.
**
** Note:            This is used ONLY to free kernel memory allocated from
**                  within the kernel portion of the NAL.  No IOCTL call will
**                  resolve to this.
****************************************************************************/
void
nal_free_memory(
    void*         Address
    )
{
    if(Address != NULL)
    {
        kfree(Address);
    }
}

/***************************************************************************
**
** Name:            nal_free_memory_non_paged_pci()
**
** Description:     Frees the memory allocated at the buffer address.
**
** Author:          AS
**
** Born on Date:    03/20/2009
**
** Arguments:       Address  = Pointer to the address to free.
**
** Returns:         Nothing.
**
****************************************************************************/
void
nal_free_memory_non_paged_pci(
    void*                  PDev,
    void*                  Address,
    unsigned long long     PhysicalAddress,
    unsigned int           Size
    )
{
    struct pci_dev*     PciDevice  = (struct pci_dev*)PDev;

    if(PDev != NULL && Address != NULL && PhysicalAddress != 0)
    {
        pci_free_consistent(PciDevice, Size, Address, PhysicalAddress);
    }
}

/***************************************************************************
**
** Name:            nal_get_physical_memory_address()
**
** Description:     Returns the Physical address associated with this virtual
**                  address. This address can be used for DMA by HW.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Address  = Pointer to the address to resolve.
**
** Returns:         Physical Address.
**
** Note:            EFI maps physical addresses to their virtual addresses,
**                  so this returns the virtual address casted.
**
****************************************************************************/
unsigned long long
nal_get_physical_memory_address(
    void* VirtualAddress
    )
{
    unsigned long long NalPhysical = 0;

    if(VirtualAddress != NULL)
    {
        NalPhysical = __pa(VirtualAddress);
    }

    return NalPhysical;
}

/***************************************************************************
**
** Name:            nal_kmem_set()
**
** Description:     Sets Dest to the value of Value for Size length.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Dest     = Pointer to the address to set
**                  Value    = Integer value to set.
**                  Size     = Bytes to set
**
** Returns:         Dest
**
****************************************************************************/
void*
nal_kmem_set(
    void*          Dest,
    int            Value,
    unsigned long  Size
    )
{
    return memset(Dest, Value, Size);
}

/***************************************************************************
**
** Name:            nal_ktou_memcpy()
**
** Description:     Copies a buffer from Kernel to User space of Size bytes.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Source     = Pointer to the Kernel Address.
**                  Dest       = Pointer to the User Address.
**                  Size       = Bytes to Copy
**
** Returns:         Dest
**
****************************************************************************/
void*
nal_ktou_memcpy(
    void*          Dest,
    void*          Source,
    unsigned long  Size
    )
{
    unsigned long  BytesLeft   = 0;
    BytesLeft = copy_to_user(Dest, Source, Size);
    return Dest;
}

/***************************************************************************
**
** Name:            nal_ktok_memcpy()
**
** Description:     Does a kernel mode memcpy between two kernel mode buffers.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Source     = Pointer to the Kernel Address.
**                  Dest       = Pointer to the Kenerl Address.
**                  Size       = Bytes to Copy
**
** Returns:         Dest
**
****************************************************************************/
void*
nal_ktok_memcpy(
    void*          Dest,
    void*          Source,
    unsigned long  Size
    )
{
    return memcpy(Dest, Source, Size);
}

/***************************************************************************
**
** Name:            nal_utok_memcpy()
**
** Description:     Copies a buffer from User to Kernel space of Size bytes.
**
** Author:          MVM
**
** Born on Date:    03/11/2003
**
** Arguments:       Source     = Pointer to the User Address.
**                  Dest       = Pointer to the Kernel Address.
**                  Size       = Bytes to Copy
**
** Returns:         Dest
**
****************************************************************************/
void*
nal_utok_memcpy(
    void*          Dest,
    void*          Source,
    unsigned long  Size
    )
{
    unsigned long  BytesLeft   = 0;
    BytesLeft = copy_from_user(Dest, Source, Size);
    return Dest;
}

/***************************************************************************
**
** Name:            nal_debug_print()
**
** Description:     Sends a text string out to log system.
**
** Author:          MVM
**
** Born on Date:    04/10/2001
**
** Arguments:       same as printf()
**
** Returns:         on success: NAL_LINUX_DRIVER_SUCCESS
**                  on failure: NAL_LINUX_DRIVER_FAILED
**
****************************************************************************/
int
nal_debug_print(
    char* Format,
    ...
    )
{
    char     FormattedStatus[500];
    va_list  ArgPtr;
    int      DbgPrintStatus = NAL_LINUX_DRIVER_FAILED;

    if(Global_DebugPrintEnabled == 1)
    {
        /* Generate the formatted debug message. */
        va_start(ArgPtr, Format);
        vsprintf(FormattedStatus, Format, ArgPtr);
        va_end(ArgPtr);

        /* Print the message to the file and set the correct status. */
        printk(KERN_DEBUG "%s", FormattedStatus);
        DbgPrintStatus = NAL_LINUX_DRIVER_SUCCESS;
    }
    return DbgPrintStatus;
}

/***************************************************************************
**
** Name:            nal_enable_debug_print()
**
** Description:     Sets Global_DebugPrintEnabled flag to whatever the
**                  value of "Enable" is currently set to.
**
** Author:          SPD
**
** Born on Date:    09/03/2002
**
** Arguments:       Enable      - Boolean flag that when set to TRUE, enables
**                                debug print support.
**
** Returns:         Nothing
**
****************************************************************************/
void
nal_enable_debug_print(
    unsigned char Enable
    )
{
    Global_DebugPrintEnabled = Enable;
}

