/*********************************************************************/
/*                     Intel Confidential                            */
/*********************************************************************/

/**********************************************************************
**                                                                   **
**                I N T E L   P R O P R I E T A R Y                  **
**                                                                   **
**   COPYRIGHT (c)  2001 - 2011  BY  INTEL  CORPORATION.       ALL   **
**   RIGHTS RESERVED.   NO PART OF THIS PROGRAM OR PUBLICATION MAY   **
**   BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A   **
**   RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER   **
**   LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,   **
**   MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT   **
**   THE PRIOR WRITTEN PERMISSION OF :                               **
**                                                                   **
**                      INTEL  CORPORATION                           **
**                                                                   **
**                2200 MISSION COLLEGE BOULEVARD                     **
**                                                                   **
**             SANTA  CLARA,  CALIFORNIA  95052-8119                 **
**                                                                   **
**********************************************************************/

/**********************************************************************
**                                                                   **
**  INTEL CORPORATION PROPRIETARY INFORMATION                        **
**                                                                   **
**  This software is supplied under the terms of a license agreement **
**  with Intel Corporation and may not be copied nor disclosed       **
**  except in accordance with the terms of that agreement.           **
**                                                                   **
**                                                                   **
**  Module Name:                                                     **
**    linuxdriverpci_i.h                                             **
**                                                                   **
**  Abstract:                                                        **
**    This file contains functions declarations for NAL PCI          **
**    functions for the Linux Driver portion.  These routines        **
**    are called as IOCTL's from the NalResolveIoctl routines.       **
**                                                                   **
***********************************************************************
* VSS Revision Control Information:
* ---------------------------------
*   $Workfile: linuxdriverpci_i.c $
*   $Date: 2011/06/20 13:26:27 $
*   $Archive: /QV2.0/nal/src/linux/driver/linuxdriverpci_i.c $
*   $Revision: 1.3 $
**********************************************************************/

#ifndef _LINUXDRIVERPCI_I_H_
#define _LINUXDRIVERPCI_I_H_


#define NAL_LINUX_KERNEL_IORESOURCE_IO         0x1
#define NAL_LINUX_KERNEL_IORESOURCE_MEM        0x2

#define NAL_LINUX_KERNEL_GET_RESOURCE_FLAG(FlagsIn, FlagsOut) \
    (FlagsIn & IORESOURCE_IO)  ? (FlagsOut |= NAL_LINUX_KERNEL_IORESOURCE_IO)  : FlagsOut; \
    (FlagsIn & IORESOURCE_MEM) ? (FlagsOut |= NAL_LINUX_KERNEL_IORESOURCE_MEM) : FlagsOut

void*
nal_pci_get_slot(
    unsigned int         Segment,
    unsigned int         Bus,
    unsigned int         Device,
    unsigned int         Function
    );

void
nal_pci_disable_device(
    void*               PciDevice
    );

int
nal_pci_enable_device(
    void*          PciDevice
    );

int
nal_pci_read_config_byte(
    void*          PciDevice,
    int            Offset,
    unsigned char *Value
    );

int
nal_pci_read_config_word(
    void*          PciDevice,
    int            Offset,
    unsigned short *Value
    );

int
nal_pci_read_config_dword(
    void*          PciDevice,
    int            Offset,
    unsigned int*  Value
    );

int
nal_pci_write_config_byte(
    void*          PciDevice,
    int            Offset,
    unsigned char  Value
    );

int
nal_pci_write_config_word(
    void*          PciDevice,
    int            Offset,
    unsigned short Value
    );

int
nal_pci_write_config_dword(
    void*          PciDevice,
    int            Offset,
    unsigned int   Value
    );

void
nal_pci_dev_put(
    void*          PciDevice
    );

int
nal_pci_set_dma_mask(
    void*               PciDevice,
    unsigned long long  mask
    );

int
nal_pci_set_consistent_dma_mask(
    void*               PciDevice,
    unsigned long long  mask
    );

void*
nal_pci_alloc_consistent(
    void*               PciDevice,
    unsigned int        Size,
    unsigned long long* DmaHandle
    );

int
nal_pci_dma_supported(
    void*               PciDevice,
    unsigned long long  Mask
    );

void
nal_pci_set_master(
    void*               PciDevice
    );

int
nal_pci_resource_start_end(
    void*                PciDevice,
    unsigned int         Bar,
    unsigned int*        Start,
    unsigned int*        End
    );

int
nal_pci_resource_flags(
    void*                PciDevice,
    unsigned int         Bar
    );

void*
nal_request_region(
    unsigned long        Start,
    unsigned long        Len,
    char*                Name
    );

void*
nal_request_mem_region(
    unsigned long        Start,
    unsigned long        Len,
    char*                Name
    );

void
nal_release_region(
    unsigned long        Start,
    unsigned long        Len
    );

void
nal_release_mem_region(
    unsigned long        Start,
    unsigned long        Len
    );

int
nal_check_mem_region(
    unsigned long        Start,
    unsigned long        Len
    );

#endif /* _LINUXDRIVERPCI_I_H_ */
